Imports System
Imports System.IO
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Resources
Imports System.Data
Imports System.Globalization
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports CADImport
Imports CADImport.CADImportForms
Imports CADImport.FaceModule
Imports CADImport.RasterImage
Imports CADImport.HPGL2
Imports CADImport.Export.DirectCADtoDXF

''' <summary>
''' Represents the main application form where CAD image and added CAD entities can be viewed.
''' </summary>
Public Class MainForm
    Inherits Form
    ' Methods
    ''' <summary>
    ''' Initializes a new instance of the <see cref="!:AddEntityDemoApplication.AddEntityForm">AddEntityForm</see> class.
    ''' </summary>
    Public Sub New()
        Me.InitParams()
        Me.InitializeComponent()
        Me.InitExtParam()
    End Sub

    <System.STAThread()> _
   Public Shared Sub Main()
        Application.EnableVisualStyles()
        Application.DoEvents()
        Application.Run(New MainForm)
    End Sub

    Private Sub addEntity_Click(ByVal sender As Object, ByVal e As EventArgs)
        Dim val As Integer = CType(sender, MenuItem).Index
        Me.numlastadd = val
        Me.ShowAddEntityDialog(val)
    End Sub

    Private Sub AddRecordToLayerList(ByVal name As String, ByVal color As Color, ByVal visible As Boolean, ByVal freeze As Boolean)
        Dim lstItem As New ListItem
        lstItem.Text = name
        Me.lForm.LayerList.Items.Add(lstItem)
        lstItem = New ListItem
        If visible Then
            lstItem.ImageIndex = 7
        Else
            lstItem.ImageIndex = 8
        End If
        Me.lForm.LayerList.Items.Add(lstItem)
        lstItem = New ListItem
        If freeze Then
            lstItem.ImageIndex = 3
        Else
            lstItem.ImageIndex = 4
        End If
        Me.lForm.LayerList.Items.Add(lstItem)
        lstItem = New ListItem
        lstItem.DrawColorRect = True
        lstItem.BackColor = color
        Me.lForm.LayerList.Items.Add(lstItem)
    End Sub

    ''' <summary>
    ''' Paints the CAD image background in the black color.
    ''' </summary>
    Public Sub Black_Click()
        Me.tlbTool.Buttons.Item(3).Pushed = False
        Me.tlbTool.Buttons.Item(4).Pushed = True
        Me.cadPictBox.BackColor = Color.Black
        Me.cadImage.DefaultColor = Color.White
        Me.cadImage.BackgroundColor = Color.Black
        If (Not Me.clipRectangle Is Nothing) Then
            Me.clipRectangle.Color = Color.White
        End If
    End Sub

    Private Sub cadPictBox_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs)
        If ((e.Button = MouseButtons.Right) OrElse (e.Button = MouseButtons.Middle)) Then
            Form.ActiveForm.Cursor = Cursors.Hand
            Me.cX = e.X
            Me.cY = e.Y
            Me.detRMouseDown = True
        ElseIf (Not Me.cadImage Is Nothing) Then
            Me.clipRectangle.EnableRect(RectangleType.Zooming, New Rectangle(e.X, e.Y, 0, 0))
        End If
    End Sub

    Private Sub cadPictBox_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
        Me.cadPictBox.Focus()
        If (Not Me.cadImage Is Nothing) Then
            If Me.detRMouseDown Then
                Me.pos.X = (Me.pos.X - (Me.cX - e.X))
                Me.pos.Y = (Me.pos.Y - (Me.cY - e.Y))
                Me.cX = e.X
                Me.cY = e.Y
                Me.cadPictBox.Invalidate()
                Me.SetPictureBoxPosition(Me.Position)
            End If
            Me.old_Pos = New PointF(CSng(e.X), CSng(e.Y))
            Dim vPt As DPoint = Me.GetRealPoint(e.X, e.Y)
            Me.stBar.Panels.Item(2).Text = String.Concat(New String() {vPt.X.ToString, ApplicationConstants.sepstr2, vPt.Y.ToString, ApplicationConstants.sepstr2, vPt.Z.ToString})
        End If
    End Sub

    Private Sub cadPictBox_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs)
        Me.detRMouseDown = False
        Me.cadPictBox.Cursor = Cursors.Default
        If (Me.clipRectangle.Enabled AndAlso (Me.clipRectangle.Type = RectangleType.Zooming)) Then
            If ((Me.clipRectangle.ClientRectangle.Width <= 10) OrElse (Me.clipRectangle.ClientRectangle.Height <= 10)) Then
                Me.cadPictBox.Invalidate()
                Return
            End If
            If (Me.sc > 1000.0!) Then
                Me.cadPictBox.Invalidate()
                Return
            End If
            Dim tmp1 As Single = (Me.visibleArea.Width / CSng(Me.clipRectangle.ClientRectangle.Width))
            Dim tmp2 As Single = (Me.visibleArea.Height / CSng(Me.clipRectangle.ClientRectangle.Height))
            If (tmp1 > tmp2) Then
                tmp1 = tmp2
            End If
            Me.pos.X = (Me.pos.X + (CSng(Me.clipRectangle.ClientRectangle.Width) / 2.0!))
            Me.pos.Y = (Me.pos.Y + (CSng(Me.clipRectangle.ClientRectangle.Height) / 2.0!))
            Me.Zoom(tmp1)
        End If
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub cadPictBox_Paint(ByVal sender As Object, ByVal e As PaintEventArgs)
        If (Not Me.cadImage Is Nothing) Then
            Me.DrawCADImage(e.Graphics)
            Me.stBar.Panels.Item(1).Text = Me.RealScale
        End If
    End Sub

    Private Sub CADPictBoxMouseWheel(ByVal sender As Object, ByVal e As MouseEventArgs)
        If (e.Delta < 0) Then
            Me.Zoom(0.7!)
        Else
            Me.Zoom(1.3!)
        End If
        Me.Shift()
        Me.SetPictureBoxPosition(Me.Position)
    End Sub

    Private Sub CADPictBoxScroll(ByVal sender As Object, ByVal e As ScrollEventArgsExt)
        Me.LeftImagePosition = -Me.cadPictBox.Position.X
        Me.TopImagePosition = -Me.cadPictBox.Position.Y
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub ChangeControlState()
        Dim detNullImg As Boolean = (Not Me.cadImage Is Nothing)
        Me.EnableButton(detNullImg)
        Dim tmp As Boolean = False
        If (Me.cadPictBox.BackColor.Equals(Color.White)) Then
            tmp = True
        End If
        If detNullImg Then
            Me.tlbWhite.Pushed = tmp
            Me.tlbBlack.Pushed = Not tmp
        Else
            Me.tlbWhite.Pushed = False
            Me.tlbBlack.Pushed = False
        End If
        Me.tlbShx.Pushed = Me.useSHXFonts
    End Sub

    Private Sub ChangeTextsType(ByVal val As Boolean)
        Me.useSHXFonts = val
        CADConst.UseSHXFonts = Me.useSHXFonts
        CADConst.UseTTFFonts = Not Me.useSHXFonts
        CADConst.UseMultyTTFFonts = Not Me.useSHXFonts
        Me.ReOpen()
    End Sub

    ''' <summary>
    ''' Makes the layers of the loaded CAD image visible or invisible.
    ''' </summary>
    ''' <param name="sender">A <see cref="T:System.Windows.Forms.CheckedListBox">CheckedListBox</see> that initializes changing a visibility of the layers.</param>
    ''' <param name="e">An <see cref="T:System.Windows.Forms.ItemCheckEventArgs">ItemCheckEventArgs</see> object that provides data for the event of changing a layer visibility.</param>
    Public Sub chLay_ItemCheck(ByVal sender As Object, ByVal e As ItemCheckEventArgs)
        If (e.NewValue = CheckState.Checked) Then
            Me.cadImage.SetLayerVisible(e.Index, True, Me.lForm.detInvalidate)
        ElseIf (e.NewValue = CheckState.Unchecked) Then
            Me.cadImage.SetLayerVisible(e.Index, False, Me.lForm.detInvalidate)
        End If
        Me.cadPictBox.Invalidate()
    End Sub

    ''' <summary>
    ''' Closes a currently open CAD file.
    ''' </summary>
    Public Sub CloseFile()
        Me.cadImage.Dispose()
        Me.cadImage = Nothing
        Me.EnableButton(False)
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub DeactCADImage()
        Me.lForm.LayerList.Clear()
        RemoveHandler Me.lForm.LayerList.SelectItemEvent, Me.listItemSelectDel
    End Sub

    ''' <summary>
    ''' Cleans up any resources being used.
    ''' </summary>
    ''' <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
    ''' </param>
    Protected Overloads Sub Dispose(ByVal disposing As Boolean)
        If (disposing AndAlso (Not Me.components Is Nothing)) Then
            Me.components.Dispose()
        End If
        MyBase.Dispose(disposing)
    End Sub

    ''' <summary>
    ''' Displays a CAD image in black and white colors.
    ''' </summary>
    Public Sub DoBlackColor()
        If (Not Me.cadImage Is Nothing) Then
            Me.cadImage.DrawMode = CADDrawMode.Black
            Me.drawMode = False
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' Resizes the CAD image to fit drawing's bounds.
    ''' </summary>
    ''' <remarks>This method is invoked when changing layout or working with 3D Orbit tool.</remarks>
    Public Sub DoResize()
        If (Not Me.cadImage Is Nothing) Then
            Dim wh As Single = CSng((Me.cadImage.AbsWidth / Me.cadImage.AbsHeight))
            Dim new_wh As Single = (CSng(Me.cadPictBox.ClientRectangle.Width) / CSng(Me.cadPictBox.ClientRectangle.Height))
            Me.visibleArea = New SizeF(Me.cadPictBox.ClientSize.Width, Me.cadPictBox.ClientSize.Height)
            If (new_wh > wh) Then
                Me.visibleArea.Width = CInt((Me.visibleArea.Height * wh))
            ElseIf (new_wh < wh) Then
                Me.visibleArea.Height = CInt((Me.visibleArea.Width / wh))
            Else
                Me.visibleArea = New SizeF(Me.cadPictBox.ClientSize.Width, Me.cadPictBox.ClientSize.Height)
            End If
            Me.pos.X = ((Me.cadPictBox.ClientRectangle.Width - Me.visibleArea.Width) / 2.0!)
            Me.pos.Y = ((Me.cadPictBox.ClientRectangle.Height - Me.visibleArea.Height) / 2.0!)
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' Increases a scale of the CAD image in two times.
    ''' </summary>
    Public Sub DoZoomIn()
        If (Not Me.cadImage Is Nothing) Then
            Me.old_Pos = New PointF((CSng(Me.cadPictBox.ClientRectangle.Width) / 2.0!), (CSng(Me.cadPictBox.ClientRectangle.Height) / 2.0!))
            Me.sc = (Me.sc * 2.0!)
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' Decreases a scale of the CAD image in two times.
    ''' </summary>
    Public Sub DoZoomOut()
        If (Not Me.cadImage Is Nothing) Then
            Me.old_Pos = New PointF((CSng(Me.cadPictBox.ClientRectangle.Width) / 2.0!), (CSng(Me.cadPictBox.ClientRectangle.Height) / 2.0!))
            Me.sc = (Me.sc / 2.0!)
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' Draws a CAD image from the loaded CAD file.
    ''' </summary>
    ''' <param name="gr">A <see cref="T:System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
    Public Sub DrawCADImage(ByVal gr As Graphics)
        Try
            Me.Shift()
            Dim tmp As RectangleF = Me.ImageRectangleF
            Me.SetSizePictureBox(New Size(CInt(tmp.Width), CInt(tmp.Height)))
            Me.cadImage.Draw(gr, tmp)
        Catch e1 As Exception
            MessageBox.Show(String.Format("[{0}] {1}", e1.Message, e1.StackTrace))
        End Try
    End Sub

    ''' <summary>
    ''' Enables or disables buttons of the toolbar and menu items of the main menu.
    ''' </summary>
    ''' <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
    ''' <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
    Public Sub EnableButton(ByVal aVal As Boolean)
        Dim i As Integer
        For i = 1 To Me.tlbTool.Buttons.Count - 1
            Me.tlbTool.Buttons.Item(i).Enabled = aVal
        Next i
        Me.miSaveAsDXF.Enabled = aVal
    End Sub

    ''' <summary>
    ''' Gets a three-dimensional point of the current CAD image from the specified screen point.
    ''' </summary>
    ''' <param name="x">An X coordinate of the screen point.</param>
    ''' <param name="y">A Y coordinate of the screen point.</param>
    ''' <returns>A three-dimensional point.</returns>
    Public Function GetRealPoint(ByVal x As Integer, ByVal y As Integer) As DPoint
        Dim tmpRect As RectangleF = Me.ImageRectangleF
        Return CADConst.GetRealPoint(x, y, Me.cadImage, tmpRect)
    End Function

    Private Sub InitExtParam()
        Me.clipRectangle = New ClipRect(Me.cadPictBox)
        AddHandler Me.cadPictBox.MouseWheel, New MouseEventHandler(AddressOf Me.CADPictBoxMouseWheel)
        AddHandler Me.cadPictBox.ScrollEvent, New ScrollEventHandlerExt(AddressOf Me.CADPictBoxScroll)
        AddHandler Me.createCircleForm.ResizeImageEvent, New CADImportEventHandler(AddressOf Me.ResizeLayout)
    End Sub

#Region " Windows Form Designer generated code "
    ''' <summary>
    ''' Required method for Designer support - do not modify
    ''' the contents of this method with the code editor.
    ''' </summary>
    Private Sub InitializeComponent()
        Me.components = New Container
        Dim resources As New ResourceManager(GetType(MainForm))
        Me.dlgOpenFile = New OpenFileDialog
        Me.stBar = New StatusBar
        Me.sbOpen = New StatusBarPanel
        Me.sbScale = New StatusBarPanel
        Me.sbCoord = New StatusBarPanel
        Me.tlbTool = New ToolBar
        Me.tlbOpen = New ToolBarButton
        Me.tlbZoomIn = New ToolBarButton
        Me.tlbZoomOut = New ToolBarButton
        Me.tlbWhite = New ToolBarButton
        Me.tlbBlack = New ToolBarButton
        Me.tlbLay = New ToolBarButton
        Me.tlbAddEnt = New ToolBarButton
        Me.addContextMenu = New ContextMenu
        Me.addCircleMenuItem = New MenuItem
        Me.addLineMenuItem = New MenuItem
        Me.addPolyLineMenuItem = New MenuItem
        Me.menuItem7 = New MenuItem
        Me.menuItem8 = New MenuItem
        Me.tlbSave = New ToolBarButton
        Me.tlbShx = New ToolBarButton
        Me.toolBtnImageList = New ImageList(Me.components)
        Me.dlgSaveImg = New SaveFileDialog
        Me.mainMenu = New MainMenu
        Me.menuItem1 = New MenuItem
        Me.miOpenFile = New MenuItem
        Me.miSaveAsDXF = New MenuItem
        Me.miSep = New MenuItem
        Me.miExit = New MenuItem
        Me.menuItem9 = New MenuItem
        Me.miSHXForm = New MenuItem
        Me.menuItem5 = New MenuItem
        Me.miRegister = New MenuItem
        Me.miFloatLic = New MenuItem
        Me.miAbout = New MenuItem
        Me.dlgSaveDXF = New SaveFileDialog
        Me.setColorDlg = New ColorDialog
        Me.cadPictBox = New CADPictureBox
        Me.sbOpen.BeginInit()
        Me.sbScale.BeginInit()
        Me.sbCoord.BeginInit()
        MyBase.SuspendLayout()
        Me.dlgOpenFile.DefaultExt = "*.dxf;*.dwg"
        Me.dlgOpenFile.Filter = "All(*.dxf,*.dwg,*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl,*.cgm, *.bmp,*.jpg, *.jpeg,*.tiff, *.gif, *.ico, *.cur, *.png, *.emf, *.wmf)|*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm|CAD files(*.dxf, *.dwg)|*.dxf;*.dwg|All raster files (*.bmp,*.jpg, *.jpeg,*.tiff,*.tif, *.gif, *.ico, *.cur, *.png)|*.bmp;*.jpg;*.jpeg;*.tiff;*.tif; *.gif;*.ico;*.cur; *.png|Metafiles ( *.emf, *.wmf)| *.emf; *.wmf|DXF files(*.dxf)|*.dxf|DWG files(*.dwg)|*.dwg|BMP(*.bmp)|*.bmp|JPG(*.jpg, *.jpeg)| *.jpg; *.jpeg|TIFF(*.tiff, *.tif)|*.tiff;*.tif|GIF(*.gif)|*.gif|ICO(*.ico)|*.ico|PNG(*.png)|*.png|Cursors(*.cur)|*.cur|HPGL(*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl)|*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl|CGM (*.cgm)|*.cgm"
        Me.dlgOpenFile.ShowHelp = True
        Me.stBar.Font = New Font("Microsoft Sans Serif", 8.0!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.stBar.Location = New Point(0, 390)
        Me.stBar.Name = "stBar"
        Me.stBar.Panels.AddRange(New StatusBarPanel() {Me.sbOpen, Me.sbScale, Me.sbCoord})
        Me.stBar.ShowPanels = True
        Me.stBar.Size = New Size(&H2AF, &H10)
        Me.stBar.TabIndex = 9
        Me.sbOpen.AutoSize = StatusBarPanelAutoSize.Contents
        Me.sbOpen.MinWidth = 100
        Me.sbOpen.ToolTipText = "File Name"
        Me.sbScale.AutoSize = StatusBarPanelAutoSize.Spring
        Me.sbScale.ToolTipText = "Scale"
        Me.sbScale.Width = 520
        Me.sbCoord.Alignment = HorizontalAlignment.Right
        Me.sbCoord.AutoSize = StatusBarPanelAutoSize.Contents
        Me.sbCoord.Text = "0 - 0 - 0"
        Me.sbCoord.Width = &H33
        Me.tlbTool.Appearance = ToolBarAppearance.Flat
        Me.tlbTool.BorderStyle = BorderStyle.FixedSingle
        Me.tlbTool.Buttons.AddRange(New ToolBarButton() {Me.tlbOpen, Me.tlbZoomIn, Me.tlbZoomOut, Me.tlbWhite, Me.tlbBlack, Me.tlbLay, Me.tlbAddEnt, Me.tlbSave, Me.tlbShx})
        Me.tlbTool.ButtonSize = New Size(20, 20)
        Me.tlbTool.DropDownArrows = True
        Me.tlbTool.ImageList = Me.toolBtnImageList
        Me.tlbTool.Location = New Point(0, 0)
        Me.tlbTool.Name = "tlbTool"
        Me.tlbTool.ShowToolTips = True
        Me.tlbTool.Size = New Size(&H2AF, &H1D)
        Me.tlbTool.TabIndex = 10
        AddHandler Me.tlbTool.ButtonClick, New ToolBarButtonClickEventHandler(AddressOf Me.tlbTool_ButtonClick)
        Me.tlbOpen.ImageIndex = 1
        Me.tlbOpen.ToolTipText = "Open file"
        Me.tlbZoomIn.Enabled = False
        Me.tlbZoomIn.ImageIndex = 2
        Me.tlbZoomIn.ToolTipText = "Zoom In(+)"
        Me.tlbZoomOut.Enabled = False
        Me.tlbZoomOut.ImageIndex = 3
        Me.tlbZoomOut.ToolTipText = "Zoom Out(-)"
        Me.tlbWhite.Enabled = False
        Me.tlbWhite.ImageIndex = 4
        Me.tlbWhite.ToolTipText = "White background"
        Me.tlbBlack.Enabled = False
        Me.tlbBlack.ImageIndex = 5
        Me.tlbBlack.ToolTipText = "Black background"
        Me.tlbLay.Enabled = False
        Me.tlbLay.ImageIndex = 6
        Me.tlbLay.ToolTipText = "Show layers form"
        Me.tlbAddEnt.DropDownMenu = Me.addContextMenu
        Me.tlbAddEnt.Enabled = False
        Me.tlbAddEnt.ImageIndex = 0
        Me.tlbAddEnt.Style = ToolBarButtonStyle.DropDownButton
        Me.tlbAddEnt.ToolTipText = "Add entity"
        Me.addContextMenu.MenuItems.AddRange(New MenuItem() {Me.addCircleMenuItem, Me.addLineMenuItem, Me.addPolyLineMenuItem, Me.menuItem7, Me.menuItem8})
        Me.addCircleMenuItem.Index = 0
        Me.addCircleMenuItem.Text = "Circle"
        AddHandler Me.addCircleMenuItem.Click, New System.EventHandler(AddressOf Me.addEntity_Click)
        Me.addLineMenuItem.Index = 1
        Me.addLineMenuItem.Text = "Line"
        AddHandler Me.addLineMenuItem.Click, New System.EventHandler(AddressOf Me.addEntity_Click)
        Me.addPolyLineMenuItem.Index = 2
        Me.addPolyLineMenuItem.Text = "Polyline"
        AddHandler Me.addPolyLineMenuItem.Click, New System.EventHandler(AddressOf Me.addEntity_Click)
        Me.menuItem7.Index = 3
        Me.menuItem7.Text = "Text"
        AddHandler Me.menuItem7.Click, New System.EventHandler(AddressOf Me.addEntity_Click)
        Me.menuItem8.Index = 4
        Me.menuItem8.Text = "MText"
        AddHandler Me.menuItem8.Click, New System.EventHandler(AddressOf Me.addEntity_Click)
        Me.tlbSave.Enabled = False
        Me.tlbSave.ImageIndex = 7
        Me.tlbSave.ToolTipText = "Save image"
        Me.tlbShx.Enabled = False
        Me.tlbShx.ImageIndex = 8
        Me.tlbShx.Pushed = True
        Me.toolBtnImageList.ColorDepth = ColorDepth.Depth32Bit
        Me.toolBtnImageList.ImageSize = New Size(&H10, &H10)
        Me.toolBtnImageList.ImageStream = DirectCast(resources.GetObject("toolBtnImageList.ImageStream"), ImageListStreamer)
        Me.toolBtnImageList.TransparentColor = Color.White
        Me.dlgSaveImg.Filter = "*.bmp|*.bmp|*.jpg|*.jpg|*.tiff|*.tiff|*.gif|*.gif|*.emf|*.emf|*.dxf|*.dxf"
        Me.dlgSaveImg.RestoreDirectory = True
        Me.mainMenu.MenuItems.AddRange(New MenuItem() {Me.menuItem1, Me.menuItem9, Me.menuItem5})
        Me.menuItem1.Index = 0
        Me.menuItem1.MenuItems.AddRange(New MenuItem() {Me.miOpenFile, Me.miSaveAsDXF, Me.miSep, Me.miExit})
        Me.menuItem1.Text = "File"
        Me.miOpenFile.Index = 0
        Me.miOpenFile.Text = "Open File ..."
        AddHandler Me.miOpenFile.Click, New System.EventHandler(AddressOf Me.miOpenFile_Click)
        Me.miSaveAsDXF.Enabled = False
        Me.miSaveAsDXF.Index = 1
        Me.miSaveAsDXF.Text = "Save as DXF..."
        AddHandler Me.miSaveAsDXF.Click, New System.EventHandler(AddressOf Me.miSaveAsDXF_Click)
        Me.miSep.Index = 2
        Me.miSep.Text = "-"
        Me.miExit.Index = 3
        Me.miExit.Text = "Close"
        AddHandler Me.miExit.Click, New System.EventHandler(AddressOf Me.miExit_Click)
        Me.menuItem9.Index = 1
        Me.menuItem9.MenuItems.AddRange(New MenuItem() {Me.miSHXForm})
        Me.menuItem9.Text = "View"
        Me.miSHXForm.Index = 0
        Me.miSHXForm.Text = "SHX Fonts"
        AddHandler Me.miSHXForm.Click, New System.EventHandler(AddressOf Me.miSHXForm_Click)
        Me.menuItem5.Index = 2
        Me.menuItem5.MenuItems.AddRange(New MenuItem() {Me.miRegister, Me.miFloatLic, Me.miAbout})
        Me.menuItem5.Text = "?"
        Me.miRegister.Index = 0
        Me.miRegister.Text = "Register"
        AddHandler Me.miRegister.Click, New System.EventHandler(AddressOf Me.miRegister_Click)
        Me.miFloatLic.Index = 1
        Me.miFloatLic.Text = "Floating License Registration..."
        AddHandler Me.miFloatLic.Click, New System.EventHandler(AddressOf Me.miFloatLic_Click)
        Me.miAbout.Index = 2
        Me.miAbout.Text = "About"
        AddHandler Me.miAbout.Click, New System.EventHandler(AddressOf Me.miAbout_Click)
        Me.dlgSaveDXF.DefaultExt = "*.dxf"
        Me.dlgSaveDXF.Filter = "*.dxf|*.dxf"
        Me.cadPictBox.BackColor = Color.Black
        Me.cadPictBox.Dock = DockStyle.Fill
        Me.cadPictBox.DoubleBuffering = True
        Me.cadPictBox.Location = New Point(0, &H1D)
        Me.cadPictBox.Name = "cadPictBox"
        Me.cadPictBox.Position = New Point(0, 0)
        Me.cadPictBox.Size = New Size(&H2AF, &H169)
        Me.cadPictBox.TabIndex = 11
        Me.cadPictBox.TabStop = False
        Me.cadPictBox.VirtualSize = New Size(0, 0)
        AddHandler Me.cadPictBox.MouseUp, New MouseEventHandler(AddressOf Me.cadPictBox_MouseUp)
        AddHandler Me.cadPictBox.Paint, New PaintEventHandler(AddressOf Me.cadPictBox_Paint)
        AddHandler Me.cadPictBox.MouseMove, New MouseEventHandler(AddressOf Me.cadPictBox_MouseMove)
        AddHandler Me.cadPictBox.MouseDown, New MouseEventHandler(AddressOf Me.cadPictBox_MouseDown)
        Me.AutoScaleBaseSize = New Size(5, 13)
        Me.BackColor = SystemColors.Control
        MyBase.ClientSize = New Size(&H2AF, &H196)
        MyBase.Controls.Add(Me.cadPictBox)
        MyBase.Controls.Add(Me.tlbTool)
        MyBase.Controls.Add(Me.stBar)
        MyBase.Menu = Me.mainMenu
        MyBase.Name = "AddEntityForm"
        Me.RightToLeft = RightToLeft.No
        MyBase.StartPosition = FormStartPosition.CenterScreen
        Me.Text = "CADImport.NET - Add Entity"
        MyBase.WindowState = FormWindowState.Maximized
        AddHandler MyBase.Resize, New System.EventHandler(AddressOf Me.MainForm_Resize)
        AddHandler MyBase.Closing, New CancelEventHandler(AddressOf Me.MainForm_Closing)
        AddHandler MyBase.Load, New System.EventHandler(AddressOf Me.MainForm_Load)
        AddHandler MyBase.Deactivate, New System.EventHandler(AddressOf Me.MainForm_Deactivate)
        Me.sbOpen.EndInit()
        Me.sbScale.EndInit()
        Me.sbCoord.EndInit()
        MyBase.ResumeLayout(False)
    End Sub
#End Region

    Private Sub InitParams()
        Me.sc = 1.0!
        Me.useSHXFonts = True
        Me.numlastadd = 0
        Me.listItemSelectDel = New ListItemSelectEventHandler(AddressOf Me.ListSelect)
        Me.old_Pos = PointF.Empty
        Me.lForm = New LayerForm
        Me.aboutFrm = New AboutForm
        Me.rasterSizeForm = New SetRasterSizeForm
        Me.createCircleForm = New AddCircleForm
        Me.createLineForm = New AddLineForm
        Me.createPolyForm = New AddPolyForm
        Me.createTextForm = New AddTextForm
        Me.createMTextForm = New AddMTextForm
        Me.frmSHX = New SHXForm
        Me.floatLicFrm = New FloatLicForm
        AddHandler Me.floatLicFrm.Change, New System.EventHandler(AddressOf Me.InvalidateImage)
        Me.regFrm = New RegForm
    End Sub

    Private Sub InvalidateImage(ByVal sender As Object, ByVal e As EventArgs)
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub ListSelect(ByVal item As ArrayList, ByVal e As ListEventSelectArgs)
        Dim itm1 As ListItem = Me.lForm.LayerList.Items.Item(e.ItemIndex)
        Dim index As Integer = Math.Floor(e.ItemIndex / Me.lForm.LayerList.Columns.Count)
        Select Case itm1.ImageIndex
            Case 3
                itm1.ImageIndex = 4
                Me.SetFreezeLayer(False, index)
                Exit Select
            Case 4
                itm1.ImageIndex = 3
                Me.SetFreezeLayer(True, index)
                Exit Select
            Case 7
                itm1.ImageIndex = 8
                Me.SetVisibleLayer(False, index)
                Exit Select
            Case 8
                itm1.ImageIndex = 7
                Me.SetVisibleLayer(True, index)
                Exit Select
        End Select
        If itm1.DrawColorRect Then
            Me.setColorDlg.Color = itm1.BackColor
            If (Me.setColorDlg.ShowDialog = DialogResult.OK) Then
                Me.SetColorLayer(Me.setColorDlg.Color, index)
                itm1.BackColor = Me.setColorDlg.Color
                Me.lForm.LayerList.InvalidateCell(e.ItemIndex)
            End If
        End If
        Me.lForm.LayerList.InvalidateCell(e.ItemIndex)
    End Sub

    ''' <summary>
    ''' Loads a CAD file.
    ''' </summary>
    Public Sub LoadFile(ByVal fileName As String)
        If (Not fileName Is Nothing) Then
            If (Not Me.cadImage Is Nothing) Then
                Me.cadImage.Dispose()
                Me.cadImage = Nothing
                Me.lForm.LayerList.Clear()
                RemoveHandler Me.lForm.LayerList.SelectItemEvent, Me.listItemSelectDel
            End If
            Me.Cursor = Cursors.WaitCursor
            Me.stBar.Panels.Item(0).Text = ApplicationConstants.loadfilestr
            Me.ImageScale = 1.0!
            Me.ImagePreviousScale = 1.0!
            Me.Position = New PointF
            Try
                Me.cadImage = cadImage.CreateImageByExtension(fileName)
            Catch ex As Exception
                MessageBox.Show(String.Format("[{0}] {1}", ex.Message, ex.StackTrace), ApplicationConstants.msgBoxDebugCaption, MessageBoxButtons.OK)
            End Try
            If TypeOf Me.cadImage Is CADRasterImage Then
                CType(Me.cadImage, CADRasterImage).Control = Me.cadPictBox
            End If
        End If
        If (Not Me.cadImage Is Nothing) Then
            Try
                If CADConst.IsWebPath(fileName) Then
                    Me.cadImage.LoadFromWeb(fileName)
                Else
                    Me.cadImage.LoadFromFile(fileName)
                End If
            Catch ex As Exception
                MessageBox.Show(String.Format("[{0}] {1}", ex.Message, ex.StackTrace), ApplicationConstants.msgBoxDebugCaption, MessageBoxButtons.OK)
            End Try
        End If
        Me.stBar.Panels.Item(0).Text = (Path.GetFileName(fileName) & ApplicationConstants.sepstr)
        cadImage.LastLoadedFilePath = Path.GetDirectoryName(fileName)
        Me.SetCADImageOptions()
    End Sub

    Private Shared Sub LoadParams(ByVal args As String())
        If (args.Length <> 0) Then
            MainForm.openFileName = args(0)
        End If
    End Sub

    Private Sub MainForm_Closing(ByVal sender As Object, ByVal e As CancelEventArgs)
        If (Protection.LicenseType = LicenseType.Floating) Then
            Me.floatLicFrm.DisableFloating()
        Else
            Protection.CloseApplication()
        End If
    End Sub

    Private Sub MainForm_Deactivate(ByVal sender As Object, ByVal e As EventArgs)
        Me.detRMouseDown = False
    End Sub

    Private Sub MainForm_Load(ByVal sender As Object, ByVal e As EventArgs)
        Me.frmSHX.AddDefaultSHXPaths()
        Me.visibleArea = New SizeF(Me.cadPictBox.ClientSize.Width, Me.cadPictBox.ClientSize.Height)
        If (Protection.LicenseType = LicenseType.Floating) Then
            Me.floatLicFrm.FloatingLicRegister()
        Else
            Protection.Register()
        End If
        If (Not MainForm.openFileName Is Nothing) Then
            Me.LoadFile(MainForm.openFileName)
        End If
        Me.ChangeControlState()
    End Sub

    Private Sub MainForm_Resize(ByVal sender As Object, ByVal e As EventArgs)
        Me.DoResize()
    End Sub

    Private Sub miAbout_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.aboutFrm.ShowDialog()
    End Sub

    Private Sub miExit_Click(ByVal sender As Object, ByVal e As EventArgs)
        MyBase.Close()
    End Sub

    Private Sub miFloatLic_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.floatLicFrm.ShowDialog()
    End Sub

    Private Sub miOpenFile_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.ShowLoadFileDialog()
        Me.ChangeControlState()
    End Sub

    Private Sub miRegister_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.regFrm.ShowDialog()
    End Sub

    Private Sub miSaveAsDXF_Click(ByVal sender As Object, ByVal e As EventArgs)
        If ((Not Me.cadImage Is Nothing) AndAlso (Me.dlgSaveDXF.ShowDialog = DialogResult.OK)) Then
            Me.SaveAsDXF(Me.dlgSaveDXF.FileName)
        End If
    End Sub

    Private Sub miSHXForm_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.frmSHX.ShowDialog()
    End Sub

    Private Sub NotSupportedInCurrentVersion()
        Me.Cursor = Cursors.Default
        Me.cadPictBox.Visible = True
        Me.stBar.Panels.Item(0).Text = ApplicationConstants.notsupportedstr
        MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr)
    End Sub

    Private Sub Open_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.ShowLoadFileDialog()
        Me.ChangeControlState()
    End Sub

    ''' <summary>
    ''' Reopens a file after selecting or deselecting SHX fonts.
    ''' </summary>
    ''' <remarks>This method is also called when the paths to the 
    ''' files containing SHX fonts have been edited in the <b>SHX Paths</b> window 
    ''' invoked by the <b>SHX Fonts</b> menu item.</remarks>
    Public Sub ReOpen()
        If (Not Me.dlgOpenFile.FileName Is Nothing) Then
            Me.LoadFile(Me.dlgOpenFile.FileName)
        End If
    End Sub

    ''' <summary>
    ''' Sets the scale and position of the CAD image to its original values.
    ''' </summary>
    Public Sub ResetScaling()
        Me.sc = 1.0!
        Me.prev_sc = 1.0!
        Me.pos.X = ((Me.cadPictBox.ClientRectangle.Width - Me.visibleArea.Width) / 2.0!)
        Me.pos.Y = ((Me.cadPictBox.ClientRectangle.Height - Me.visibleArea.Height) / 2.0!)
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub ResizeLayout()
        If (Not Me.cadImage Is Nothing) Then
            Me.cadImage.GetExtents()
            Me.DoResize()
        End If
    End Sub

    Private Sub SaveAsDXF(ByVal fName As String)
        Dim exp1 As CADtoDXF
        If (Not Me.cadImage Is Nothing) Then
            If TypeOf Me.cadImage Is CADRasterImage Then
                If ((Not Me.cadImage.FileInf Is Nothing) AndAlso (Me.cadImage.FileInf.Extension.ToUpper = ApplicationConstants.emfextstr)) Then
                    Me.cadImage.EmfConvertToDXF(Me.cadImage.FileInf.FullName, Me.cadPictBox.Handle, fName)
                End If
            Else
                exp1 = New CADtoDXF(Me.cadImage)
                exp1.SaveToFile(fName)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Invokes a <see cref="T:System.Windows.Forms.SaveFileDialog">SaveFileDialog</see> box where 
    ''' a user can select a directory and format for saving the current CAD image.
    ''' </summary>
    ''' <remarks><p>This method can be invoked from the toolbar and from the main menu.</p>
    ''' <p>Images can be saved in the following formats: JPG, BMP, TIFF, GIF, EMF.</p></remarks>
    Public Sub SaveAsImage()
        If (Not Me.cadImage Is Nothing) Then
            If (Not Me.cadImage.FileInf Is Nothing) Then
                Me.dlgSaveImg.FileName = Me.cadImage.FileInf.Name.Replace(Me.cadImage.FileInf.Extension, String.Empty)
            End If
            If (Me.dlgSaveImg.ShowDialog = DialogResult.OK) Then
                Dim tmpRect As New DRect(0, 0, CDbl((Me.VisibleAreaSize.Width * Me.ImageScale)), CDbl((Me.VisibleAreaSize.Height * Me.ImageScale)))
                Dim imgFrm As ImageFormat = ImageFormat.Bmp
                If (Not Me.dlgSaveImg.FileName Is Nothing) Then
                    Dim tmp As String = Me.dlgSaveImg.FileName
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.jpgextstr) <> -1) Then
                        imgFrm = ImageFormat.Jpeg
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.bmpextstr) <> -1) Then
                        imgFrm = ImageFormat.Bmp
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.tiffextstr) <> -1) Then
                        imgFrm = ImageFormat.Tiff
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.gifextstr) <> -1) Then
                        imgFrm = ImageFormat.Gif
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.emfextstr) <> -1) Then
                        imgFrm = ImageFormat.Emf
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.wmfextstr) <> -1) Then
                        imgFrm = ImageFormat.Wmf
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.pngextstr) <> -1) Then
                        imgFrm = ImageFormat.Png
                    End If
                    If (tmp.ToUpper.IndexOf(ApplicationConstants.dxfextstr) <> -1) Then
                        Me.SaveAsDXF(Me.dlgSaveImg.FileName)
                        Return
                    End If
                End If
                If Not Me.clipRectangle.Enabled Then
                    Me.rasterSizeForm.Image = Me.cadImage
                    Me.rasterSizeForm.CurrentSize = tmpRect
                    Me.rasterSizeForm.SaveFileName = Me.dlgSaveImg.FileName
                    If (Me.rasterSizeForm.ShowDialog = DialogResult.OK) Then
                        tmpRect = Me.rasterSizeForm.SizeImage
                    End If
                End If
                Dim tmpNumberOfPartsInCircle As Integer = Me.cadImage.NumberOfPartsInCircle
                Me.cadImage.NumberOfPartsInCircle = CADConst.SetNumberOfPartsInCurve(tmpRect)
                If Me.clipRectangle.Enabled Then
                    If ((imgFrm Is ImageFormat.Emf) OrElse (imgFrm Is ImageFormat.Wmf)) Then
                        Me.cadImage.ExportToMetafile(Me.dlgSaveImg.FileName, tmpRect, Me.clipRectangle.ClientRectangle)
                    Else
                        Me.cadImage.SaveToFile(Me.dlgSaveImg.FileName, imgFrm, tmpRect, Me.clipRectangle.ClientRectangle, Me.rasterSizeForm.ImagePixelFormat)
                    End If
                ElseIf ((imgFrm Is ImageFormat.Emf) OrElse (imgFrm Is ImageFormat.Wmf)) Then
                    Me.cadImage.ExportToMetafile(Me.dlgSaveImg.FileName, tmpRect)
                Else
                    Me.cadImage.SaveToFile(Me.dlgSaveImg.FileName, imgFrm, tmpRect, Me.rasterSizeForm.ImagePixelFormat)
                End If
                Me.cadImage.NumberOfPartsInCircle = tmpNumberOfPartsInCircle
            End If
        End If
    End Sub

    Public Sub SetCADImageOptions()
        Me.cadImage.UseWinEllipse = False
        Me.cadImage.IsShowLineWeight = True
        Me.cadImage.DrawMode = CADDrawMode.Normal
        Me.cadImage.UseDoubleBuffering = False
        If (Me.cadPictBox.BackColor.Equals(Color.White)) Then
            Me.White_Click()
        Else
            Me.Black_Click()
        End If
        Me.Cursor = Cursors.Default
        Me.SetLayList()
        ObjEntity.cadImage = Me.cadImage
        Me.DoResize()
        Me.SetPictureBoxPosition(Me.Position)
        Me.createCircleForm.Image = Me.cadImage
        Me.createLineForm.Image = Me.cadImage
        Me.createPolyForm.Image = Me.cadImage
        Me.createTextForm.Image = Me.cadImage
        Me.createMTextForm.Image = Me.cadImage
    End Sub

    Public Sub SetColorLayer(ByVal color As Color, ByVal index As Integer)
        Me.cadImage.SetLayerColor(index, color, Me.lForm.detInvalidate)
        Me.cadPictBox.Invalidate()
    End Sub

    Public Sub SetFreezeLayer(ByVal visible As Boolean, ByVal index As Integer)
        Me.cadImage.SetLayerFreeze(index, visible, Me.lForm.detInvalidate)
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub SetLayList()
        If ((Me.lForm.LayerList.Items.Count = 0) AndAlso (Not Me.cadImage Is Nothing)) Then
            ObjEntity.LayersList = New String(Me.cadImage.LayersCount - 1) {}
            Me.lForm.LayerList.Columns.Add(New ListHeader(ApplicationConstants.headstr1, ApplicationConstants.namestr, 0))
            Me.lForm.LayerList.Columns.Add(New ListHeader(ApplicationConstants.headstr2, ApplicationConstants.visiblestr, 0))
            Me.lForm.LayerList.Columns.Add(New ListHeader(ApplicationConstants.headstr4, ApplicationConstants.freezestr, 0))
            Me.lForm.LayerList.Columns.Add(New ListHeader(ApplicationConstants.headstr3, ApplicationConstants.colorstr, 0))
            Dim I As Integer
            For I = 0 To Me.cadImage.LayersCount - 1
                Dim tmp As String = Me.cadImage.GetLayerName(I)
                Dim colorLay As Color = Me.cadImage.GetLayerColor(I)
                ObjEntity.LayersList(I) = tmp
                Me.AddRecordToLayerList(tmp, colorLay, Me.cadImage.GetLayerVisible(I), Me.cadImage.GetLayerFreeze(I))
            Next I
            Me.lForm.LayerList.SetAutoSize()
            AddHandler Me.lForm.LayerList.SelectItemEvent, Me.listItemSelectDel
        End If
    End Sub

    Private Sub SetPictureBoxPosition(ByVal value As PointF)
        Dim w1 As Integer
        Dim h1 As Integer
        If (value.X > 0.0!) Then
            w1 = 0
        Else
            w1 = CInt(Math.Abs(value.X))
        End If
        If (w1 > Me.cadPictBox.VirtualSize.Width) Then
            w1 = Me.cadPictBox.VirtualSize.Width
        End If
        If (value.Y > 0.0!) Then
            h1 = 0
        Else
            h1 = CInt(Math.Abs(value.Y))
        End If
        If (h1 > Me.cadPictBox.VirtualSize.Height) Then
            h1 = Me.cadPictBox.VirtualSize.Height
        End If
        Me.cadPictBox.SetPositionNoInvalidate(New Point(w1, h1))
    End Sub

    Public Sub SetSizePictureBox(ByVal sz As Size)
        Me.cadPictBox.SetVirtualSizeNoInvalidate(sz)
    End Sub

    Public Sub SetVisibleLayer(ByVal visible As Boolean, ByVal index As Integer)
        Me.cadImage.SetLayerVisible(index, visible, Me.lForm.detInvalidate)
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub Shift()
        Me.pos.X = (Me.old_Pos.X - (((Me.old_Pos.X - Me.pos.X) * Me.sc) / Me.prev_sc))
        Me.pos.Y = (Me.old_Pos.Y - (((Me.old_Pos.Y - Me.pos.Y) * Me.sc) / Me.prev_sc))
        Me.prev_sc = Me.sc
    End Sub

    Private Sub ShowAddEntityDialog(ByVal num As Integer)
        Dim res As DialogResult = DialogResult.No
        Select Case num
            Case 0
                res = Me.createCircleForm.ShowDialog
                Exit Select
            Case 1
                res = Me.createLineForm.ShowDialog
                Exit Select
            Case 2
                res = Me.createPolyForm.ShowDialog
                Exit Select
            Case 3
                res = Me.createTextForm.ShowDialog
                Exit Select
            Case 4
                res = Me.createMTextForm.ShowDialog
                Exit Select
        End Select
        If (res = DialogResult.OK) Then
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    Public Sub ShowLoadFileDialog()
        Me.DeactCADImage()
        Me.cadPictBox.Enabled = False
        If (Me.dlgOpenFile.ShowDialog <> DialogResult.OK) Then
            Me.cadPictBox.Visible = True
            Me.cadPictBox.Enabled = True
        Else
            Me.cadPictBox.Enabled = True
            Me.LoadFile(Me.dlgOpenFile.FileName)
        End If
    End Sub

    Private Sub tlbTool_ButtonClick(ByVal sender As Object, ByVal e As ToolBarButtonClickEventArgs)
        If ((Not Me.cadImage Is Nothing) OrElse (Me.tlbTool.Buttons.IndexOf(e.Button) <> 8)) Then
            Select Case Me.tlbTool.Buttons.IndexOf(e.Button)
                Case 0
                    Me.ShowLoadFileDialog()
                    Exit Select
                Case 1
                    Me.DoZoomIn()
                    Exit Select
                Case 2
                    Me.DoZoomOut()
                    Exit Select
                Case 3
                    Me.White_Click()
                    Exit Select
                Case 4
                    Me.Black_Click()
                    Exit Select
                Case 5
                    Me.SetLayList()
                    Me.lForm.ShowDialog()
                    Exit Select
                Case 6
                    Me.ShowAddEntityDialog(Me.numlastadd)
                    Exit Select
                Case 7
                    Me.SaveAsImage()
                    Exit Select
                Case 8
                    If (MessageBox.Show(ApplicationConstants.msgstr, ApplicationConstants.appconst, MessageBoxButtons.YesNo) = DialogResult.Yes) Then
                        Me.ChangeTextsType(Not Me.useSHXFonts)
                    End If
                    Exit Select
            End Select
            Me.ChangeControlState()
        End If
    End Sub

    ''' <summary>
    ''' Paints the CAD image background in the white color.
    ''' </summary>
    Public Sub White_Click()
        Me.tlbTool.Buttons.Item(3).Pushed = True
        Me.tlbTool.Buttons.Item(4).Pushed = False
        Me.cadPictBox.BackColor = Color.White
        Me.cadImage.DefaultColor = Color.Black
        Me.cadImage.BackgroundColor = Color.White
        If (Not Me.clipRectangle Is Nothing) Then
            Me.clipRectangle.Color = Color.Black
        End If
    End Sub

    ''' <summary>
    ''' Adjusts the image size according to the specified scale.
    ''' </summary>
    ''' <param name="i">A scale value.</param>
    ''' <remarks>If the scale value is more than one the image size increases; otherwise, decreases.</remarks>
    Public Sub Zoom(ByVal i As Single)
        If (Not Me.cadImage Is Nothing) Then
            Me.sc = (Me.sc * i)
            If (Me.sc < 0.005!) Then
                Me.sc = 0.005!
            End If
            Me.cadPictBox.Invalidate()
            Me.cadPictBox_MouseMove(Me.cadPictBox, New MouseEventArgs(MouseButtons.Left, 0, CInt(Me.old_Pos.X), CInt(Me.old_Pos.Y), 0))
        End If
    End Sub

    Private Sub ZoomIn_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.DoZoomIn()
    End Sub

    Private Sub ZoomOut_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.DoZoomOut()
    End Sub


    ' Properties
    ''' <summary>
    ''' An X coordinate of the last mouse click position on the screen.
    ''' </summary>
    Public ReadOnly Property CurrentXClickPosition() As Integer
        Get
            Return Me.cX
        End Get
    End Property

    ''' <summary>
    ''' A Y coordinate of the last mouse click position on the screen.
    ''' </summary>
    Public ReadOnly Property CurrentYClickPosition() As Integer
        Get
            Return Me.cY
        End Get
    End Property

    Public Property DrawMode() As Boolean
        Get
            Return Me.drwMode
        End Get
        Set(ByVal value As Boolean)
            Me.drwMode = value
        End Set
    End Property

    ''' <summary>
    ''' A CAD image displayed in the main application form.
    ''' </summary>
    Public Property Image() As cadImage
        Get
            Return Me.cadImage
        End Get
        Set(ByVal value As cadImage)
            Me.cadImage = value
        End Set
    End Property

    Public ReadOnly Property ImageDRect() As DRect
        Get
            Return New DRect(CDbl(Me.LeftImagePosition), CDbl(Me.TopImagePosition), CDbl((Me.VisibleAreaSize.Width * Me.ImageScale)), CDbl((Me.VisibleAreaSize.Height * Me.ImageScale)))
        End Get
    End Property

    ''' <summary>
    ''' A previous value of the scale in which the current CAD image is displayed in the viewer.
    ''' </summary>
    Public Property ImagePreviousScale() As Single
        Get
            Return Me.prev_sc
        End Get
        Set(ByVal value As Single)
            Me.prev_sc = value
        End Set
    End Property

    Public ReadOnly Property ImageRectangleF() As RectangleF
        Get
            Return New RectangleF(Me.LeftImagePosition, Me.TopImagePosition, (Me.VisibleAreaSize.Width * Me.ImageScale), (Me.VisibleAreaSize.Height * Me.ImageScale))
        End Get
    End Property

    ''' <summary>
    ''' A scale in which the current CAD image is displayed in the viewer.
    ''' </summary>
    Public Property ImageScale() As Single
        Get
            Return Me.sc
        End Get
        Set(ByVal value As Single)
            Me.sc = value
        End Set
    End Property

    Public Property LeftImagePosition() As Single
        Get
            Return Me.pos.X
        End Get
        Set(ByVal value As Single)
            Me.pos.X = value
        End Set
    End Property

    ''' <summary>
    ''' A position of the current CAD image in relation to the top-left corner of the viewer.
    ''' </summary>
    Public Property Position() As PointF
        Get
            Return Me.pos
        End Get
        Set(ByVal value As PointF)
            Me.pos = value
        End Set
    End Property

    ''' <summary>
    ''' A previous position of the current CAD image in relation to the top-left corner of the viewer.
    ''' </summary>
    Public Property PreviousPosition() As PointF
        Get
            Return Me.old_Pos
        End Get
        Set(ByVal value As PointF)
            Me.old_Pos = value
        End Set
    End Property

    Private ReadOnly Property RealScale() As String
        Get
            If (Not Me.cadImage Is Nothing) Then
                Return String.Format("{0,2:F}%", (((CDbl((Me.VisibleAreaSize.Width * Me.ImageScale)) / Me.cadImage.AbsWidth) * Me.cadImage.MMToPixelX) * 100))
            End If
            Return String.Format("{0}%", Me.ImageScale)
        End Get
    End Property

    Public Property TopImagePosition() As Single
        Get
            Return Me.pos.Y
        End Get
        Set(ByVal value As Single)
            Me.pos.Y = value
        End Set
    End Property

    Public Property VisibleAreaSize() As SizeF
        Get
            Return Me.visibleArea
        End Get
        Set(ByVal value As SizeF)
            Me.visibleArea = value
        End Set
    End Property


    ' Fields
    Private aboutFrm As AboutForm
    Private addCircleMenuItem As MenuItem
    Private addContextMenu As ContextMenu
    Private addLineMenuItem As MenuItem
    Private addPolyLineMenuItem As MenuItem
    Private cadImage As cadImage
    Private cadPictBox As CADPictureBox
    Private clipRectangle As ClipRect
    Private components As IContainer
    Private createCircleForm As AddCircleForm
    Private createLineForm As AddLineForm
    Private createMTextForm As AddMTextForm
    Private createPolyForm As AddPolyForm
    Private createTextForm As AddTextForm
    Private cX As Integer
    Private cY As Integer
    Private detRMouseDown As Boolean
    Private dlgOpenFile As OpenFileDialog
    Private dlgSaveDXF As SaveFileDialog
    Private dlgSaveImg As SaveFileDialog
    Private drwMode As Boolean
    Private floatLicFrm As FloatLicForm
    Private frmSHX As SHXForm
    Private lForm As LayerForm
    Private listItemSelectDel As ListItemSelectEventHandler
    Private mainMenu As mainMenu
    Private menuItem1 As MenuItem
    Private menuItem5 As MenuItem
    Private menuItem7 As MenuItem
    Private menuItem8 As MenuItem
    Private menuItem9 As MenuItem
    Private miAbout As MenuItem
    Private miExit As MenuItem
    Private miFloatLic As MenuItem
    Private miOpenFile As MenuItem
    Private miRegister As MenuItem
    Private miSaveAsDXF As MenuItem
    Private miSep As MenuItem
    Private miSHXForm As MenuItem
    Private numlastadd As Integer
    Private old_Pos As PointF
    Private Shared openFileName As String
    Private pos As PointF
    Private prev_sc As Single
    Private rasterSizeForm As SetRasterSizeForm
    Private regFrm As RegForm
    Private sbCoord As StatusBarPanel
    Private sbOpen As StatusBarPanel
    Private sbScale As StatusBarPanel
    Private sc As Single
    Private setColorDlg As ColorDialog
    Private stBar As StatusBar
    Private tlbAddEnt As ToolBarButton
    Private tlbBlack As ToolBarButton
    Private tlbLay As ToolBarButton
    Private tlbOpen As ToolBarButton
    Private tlbSave As ToolBarButton
    Private tlbShx As ToolBarButton
    Private tlbTool As ToolBar
    Private tlbWhite As ToolBarButton
    Private tlbZoomIn As ToolBarButton
    Private tlbZoomOut As ToolBarButton
    Private toolBtnImageList As ImageList
    Private useSHXFonts As Boolean
    Private visibleArea As SizeF
End Class

Friend MustInherit Class ApplicationConstants
    ' Methods
    Protected Sub New()
    End Sub

    ' Fields
    Public Shared ReadOnly appconst As String = "Application"
    Public Shared ReadOnly appnamestr As String = "CADImportNet Demo"
    Public Shared ReadOnly backcolorstr As String = "BackgroundColor"
    Public Shared ReadOnly blackstr As String = "BLACK"
    Public Shared ReadOnly blackstr2 As String = "Black"
    Public Shared ReadOnly bmpextstr As String = ".BMP"
    Public Shared ReadOnly boldstr As String = "bold"
    Public Shared ReadOnly colorstr As String = "Color"
    Public Shared ReadOnly defaultstr As String = "Default"
    Public Shared ReadOnly drawmodestr As String = "drawMode"
    Public Shared ReadOnly dxfextstr As String = ".DXF"
    Public Shared ReadOnly emfextstr As String = ".EMF"
    Public Shared ReadOnly floatingstr As String = "floating"
    Public Shared ReadOnly font1str As String = "SansSerif"
    Public Shared ReadOnly freezestr As String = "Freeze"
    Public Shared ReadOnly gifextstr As String = ".GIF"
    Public Shared ReadOnly headstr1 As String = "Head1"
    Public Shared ReadOnly headstr2 As String = "Head2"
    Public Shared ReadOnly headstr3 As String = "Head3"
    Public Shared ReadOnly headstr4 As String = "Head4"
    Public Shared ReadOnly hoststr As String = "Host"
    Public Shared ReadOnly installstr As String = "Install"
    Public Shared ReadOnly italicstr As String = ", italic"
    Public Shared ReadOnly jpgextstr As String = ".JPG"
    Public Shared ReadOnly languageIDstr As String = "LanguageID"
    Public Shared ReadOnly languagepath As String = "LanguagePath"
    Public Shared ReadOnly languagestr As String = "Language"
    Public Shared ReadOnly lngextstr As String = ".lng"
    Public Shared ReadOnly loadfilestr As String = "Loading file..."
    Public Shared ReadOnly lstr As String = "l"
    Public Shared ReadOnly lstr2 As String = "L"
    Public Shared ReadOnly msgBoxDebugCaption As String = "Debug application message"
    Public Shared ReadOnly msgstr As String = "If you proceed with this operation, all added objects will be lost. Continue?"
    Public Shared ReadOnly msgstr2 As String = "Can't add a polyline with no vertices. Please, add a vertex first."
    Public Shared ReadOnly namestr As String = "Name"
    Public Shared ReadOnly notsupportedextstr As String = "Not supported in current version!"
    Public Shared ReadOnly notsupportedstr As String = "not supported"
    Public Shared ReadOnly numInfo As NumberFormatInfo = New NumberFormatInfo
    Public Shared ReadOnly onestr As String = "0.1"
    Public Shared ReadOnly onestr2 As String = "1.0"
    Public Shared ReadOnly onestr3 As String = "1"
    Public Shared ReadOnly pngextstr As String = ".PNG"
    Public Shared ReadOnly portstr As String = "Port"
    Public Shared ReadOnly registerstr As String = "register"
    Public Shared ReadOnly regularstr As String = "regular"
    Public Shared ReadOnly sepstr As String = " - "
    Public Shared ReadOnly sepstr2 As String = " : "
    Public Shared ReadOnly sepstr3 As String = ";"
    Public Shared ReadOnly sepstr4 As String = "."
    Public Shared ReadOnly showentitystr As String = "ShowEntity"
    Public Shared ReadOnly shxpathcnstr As String = "SHXPathCount"
    Public Shared ReadOnly tiffextstr As String = ".TIFF"
    Public Shared ReadOnly truestr As String = "TRUE"
    Public Shared ReadOnly typelcstr As String = "Type_license"
    Public Shared ReadOnly underlinedstr As String = ", underlined"
    Public Shared ReadOnly visiblestr As String = "Visible"
    Public Shared ReadOnly whitestr As String = "White"
    Public Shared ReadOnly wmfextstr As String = ".WMF"
    Public Shared ReadOnly zerostr As String = "0"
End Class