Imports System
Imports System.IO
Imports System.Resources
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Data
Imports System.Globalization
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports CADImport
Imports CADImport.CADImportForms
Imports CADImport.HPGL2
Imports CADImport.RasterImage
Imports CADImport.FaceModule

''' <summary>
''' Represents the main application form in which CAD image can be viewed.
''' </summary>
Public Class MainForm
    Inherits Form
    ' Methods
    ''' <summary>
    ''' Initializes a new instance of the <see cref="!:ImportDemoApplication.MainForm">MainForm</see> class.
    ''' </summary>
    Public Sub New()
        Me.InitParams()
        Me.InitializeComponent()
        Me.InitExtParam()
    End Sub

    <System.STAThread()> _
Public Shared Sub Main()
        Application.EnableVisualStyles()
        Application.DoEvents()
        Application.Run(New MainForm)
    End Sub

    Private Sub cadPictBox_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs)
        If (e.Button = MouseButtons.Right) Then
            Form.ActiveForm.Cursor = Cursors.Hand
            Me.cX = e.X
            Me.cY = e.Y
            Me.detRMouseDown = True
        End If
    End Sub

    Private Sub cadPictBox_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
        Me.cadPictBox.Focus()
        If (Not Me.cadImage Is Nothing) Then
            If Me.detRMouseDown Then
                Me.pos.X = (Me.pos.X - (Me.cX - e.X))
                Me.pos.Y = (Me.pos.Y - (Me.cY - e.Y))
                Me.cX = e.X
                Me.cY = e.Y
                Me.cadPictBox.Invalidate()
                Me.SetPictureBoxPosition(Me.Position)
            End If
            Me.PreviousPosition = New PointF(CSng(e.X), CSng(e.Y))
            Dim vPt As DPoint = Me.GetRealPoint(e.X, e.Y)
            Me.stBar.Panels.Item(2).Text = String.Concat(New String() {vPt.X.ToString, ApplicationConstants.sepstr2, vPt.Y.ToString, ApplicationConstants.sepstr2, vPt.Z.ToString})
        End If
    End Sub

    Private Sub cadPictBox_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs)
        Me.detRMouseDown = False
        Me.cadPictBox.Cursor = Cursors.Default
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub cadPictBox_Paint(ByVal sender As Object, ByVal e As PaintEventArgs)
        If (Not Me.cadImage Is Nothing) Then
            Me.DrawCADImage(e.Graphics)
            Me.stBar.Panels.Item(1).Text = Me.RealScale
        End If
    End Sub

    Private Sub CADPictBoxMouseWheel(ByVal sender As Object, ByVal e As MouseEventArgs)
        If (e.Delta < 0) Then
            Me.Zoom(0.7!)
        Else
            Me.Zoom(1.3!)
        End If
        Me.Shift()
        Me.SetPictureBoxPosition(Me.Position)
    End Sub

    Private Sub CADPictBoxScroll(ByVal sender As Object, ByVal e As ScrollEventArgsExt)
        Me.LeftImagePosition = -Me.cadPictBox.Position.X
        Me.TopImagePosition = -Me.cadPictBox.Position.Y
        Me.cadPictBox.Invalidate()
    End Sub

    Private Sub ChangeControlState()
        Dim detNullImg As Boolean = (Not Me.cadImage Is Nothing)
        Me.EnableButton(detNullImg)
    End Sub

    ''' <summary>
    ''' Resizes the CAD image to fit drawing's bounds.
    ''' </summary>
    ''' <remarks>This method is invoked when changing layout or working with a 3D Orbit tool.</remarks>
    Public Sub DoResize()
        If (Not Me.cadImage Is Nothing) Then
            Dim wh As Single = CSng((Me.cadImage.AbsWidth / Me.cadImage.AbsHeight))
            Dim new_wh As Single = (CSng(Me.cadPictBox.ClientRectangle.Width) / CSng(Me.cadPictBox.ClientRectangle.Height))
            If TypeOf Me.cadImage Is CADRasterImage Then
                Me.visibleArea = New SizeF(CSng(Me.cadImage.AbsWidth), CSng(Me.cadImage.AbsHeight))
            Else
                Me.visibleArea = New SizeF(Me.cadPictBox.Size.Width, Me.cadPictBox.Size.Height)
            End If
            If (new_wh > wh) Then
                Me.visibleArea.Width = (Me.visibleArea.Height * wh)
            ElseIf (new_wh < wh) Then
                Me.visibleArea.Height = (Me.visibleArea.Width / wh)
            Else
                Me.visibleArea = New SizeF(Me.cadPictBox.Size.Width, Me.cadPictBox.Size.Height)
            End If
            Me.LeftImagePosition = ((Me.cadPictBox.ClientRectangle.Width - Me.visibleArea.Width) / 2.0!)
            Me.TopImagePosition = ((Me.cadPictBox.ClientRectangle.Height - Me.visibleArea.Height) / 2.0!)
            Me.cadPictBox.Invalidate()
        End If
    End Sub

    ''' <summary>
    ''' Draws a CAD image from the loaded CAD file.
    ''' </summary>
    ''' <param name="gr">A <see cref="T:System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
    Public Sub DrawCADImage(ByVal gr As Graphics)
        Try
            Me.Shift()
            Dim tmp As RectangleF = Me.ImageRectangleF
            Me.SetSizePictureBox(New Size(CInt(tmp.Width), CInt(tmp.Height)))
            Me.cadImage.Draw(gr, tmp)
        Catch e1 As Exception
            MessageBox.Show(String.Format("[{0}] {1}", e1.Message, e1.StackTrace))
        End Try
    End Sub

    ''' <summary>
    ''' Enables or disables buttons of the toolbar and menu items of the main menu.
    ''' </summary>
    ''' <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
    ''' <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
    Public Sub EnableButton(ByVal aVal As Boolean)
        Me.miScale.Enabled = aVal
        Me.miSaveFile.Enabled = aVal
        Me.btnSave.Enabled = aVal
    End Sub

    ''' <summary>
    ''' Gets a three-dimensional point of the current CAD image from the specified screen point.
    ''' </summary>
    ''' <param name="x">An X coordinate of the screen point.</param>
    ''' <param name="y">A Y coordinate of the screen point.</param>
    ''' <returns>A three-dimensional point.</returns>
    Public Function GetRealPoint(ByVal x As Integer, ByVal y As Integer) As DPoint
        Dim tmpRect As RectangleF = Me.ImageRectangleF
        Return CADConst.GetRealPoint(x, y, Me.cadImage, tmpRect)
    End Function

    Private Sub InitExtParam()
        AddHandler Me.cadPictBox.MouseWheel, New MouseEventHandler(AddressOf Me.CADPictBoxMouseWheel)
        AddHandler Me.cadPictBox.ScrollEvent, New CADImport.FaceModule.ScrollEventHandlerExt(AddressOf Me.CADPictBoxScroll)
        AddHandler Me.floatLicFrm.Change, New System.EventHandler(AddressOf Me.InvalidateImage)
        Me.old_Pos = New PointF
        Me.sc = 1.0!
        Me.prev_sc = 1.0!
    End Sub

#Region " Windows Form Designer generated code "
    ''' <summary>
    ''' Required method for Designer support - do not modify
    ''' the contents of this method with the code editor.
    ''' </summary>
    Private Sub InitializeComponent()
        Dim resources As New ResourceManager(GetType(MainForm))
        Me.dlgOpenFile = New OpenFileDialog
        Me.dlgSaveImg = New SaveFileDialog
        Me.stBar = New StatusBar
        Me.sbOpen = New StatusBarPanel
        Me.sbScale = New StatusBarPanel
        Me.sbCoord = New StatusBarPanel
        Me.mainMenu = New MainMenu
        Me.miFile = New MenuItem
        Me.miOpenFile = New MenuItem
        Me.miSaveFile = New MenuItem
        Me.miSep = New MenuItem
        Me.miExit = New MenuItem
        Me.miView = New MenuItem
        Me.miScale = New MenuItem
        Me.miZoom10 = New MenuItem
        Me.miZoom25 = New MenuItem
        Me.miZoom50 = New MenuItem
        Me.miZoom100 = New MenuItem
        Me.miZoom200 = New MenuItem
        Me.miZoom400 = New MenuItem
        Me.miZoom800 = New MenuItem
        Me.miHelp = New MenuItem
        Me.miRegister = New MenuItem
        Me.miFloatLic = New MenuItem
        Me.miAbout = New MenuItem
        Me.pnlLayouts = New Panel
        Me.btnSave = New Button
        Me.btnLoad = New Button
        Me.cadPictBox = New CADPictureBox
        Me.dlgSaveEnt = New SaveFileDialog
        Me.sbOpen.BeginInit()
        Me.sbScale.BeginInit()
        Me.sbCoord.BeginInit()
        Me.pnlLayouts.SuspendLayout()
        MyBase.SuspendLayout()
        Me.dlgOpenFile.DefaultExt = "*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm"
        Me.dlgOpenFile.Filter = "All(*.dxf,*.dwg,*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl,*.cgm, *.bmp,*.jpg, *.jpeg,*.tiff, *.gif, *.ico, *.cur, *.png, *.emf, *.wmf, *.gbr)|*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm;*.gbr|CAD files(*.dxf, *.dwg)|*.dxf;*.dwg|All raster files (*.bmp,*.jpg, *.jpeg,*.tiff,*.tif, *.gif, *.ico, *.cur, *.png)|*.bmp;*.jpg;*.jpeg;*.tiff;*.tif; *.gif;*.ico;*.cur; *.png|Metafiles ( *.emf, *.wmf)| *.emf; *.wmf|DXF files(*.dxf)|*.dxf|DWG files(*.dwg)|*.dwg|BMP(*.bmp)|*.bmp|JPG(*.jpg, *.jpeg)| *.jpg; *.jpeg|TIFF(*.tiff, *.tif)|*.tiff;*.tif|GIF(*.gif)|*.gif|ICO(*.ico)|*.ico|PNG(*.png)|*.png|Cursors(*.cur)|*.cur|HPGL(*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl)|*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl|CGM (*.cgm)|*.cgm|(Gerber)*.gbr|*.gbr"
        Me.dlgOpenFile.ShowHelp = True
        Me.dlgSaveImg.Filter = "*.bmp|*.bmp|*.jpg|*.jpg|*.tiff|*.tiff|*.gif|*.gif|*.emf|*.emf"
        Me.dlgSaveImg.RestoreDirectory = True
        Me.stBar.Font = New Font("Microsoft Sans Serif", 8.0!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.stBar.Location = New Point(0, 300)
        Me.stBar.Name = "stBar"
        Me.stBar.Panels.AddRange(New StatusBarPanel() {Me.sbOpen, Me.sbScale, Me.sbCoord})
        Me.stBar.ShowPanels = True
        Me.stBar.Size = New Size(&H2AF, &H10)
        Me.stBar.TabIndex = 9
        Me.sbOpen.AutoSize = StatusBarPanelAutoSize.Contents
        Me.sbOpen.MinWidth = 100
        Me.sbOpen.ToolTipText = "File Name"
        Me.sbScale.AutoSize = StatusBarPanelAutoSize.Spring
        Me.sbScale.ToolTipText = "Scale"
        Me.sbScale.Width = 520
        Me.sbCoord.Alignment = HorizontalAlignment.Right
        Me.sbCoord.AutoSize = StatusBarPanelAutoSize.Contents
        Me.sbCoord.Text = "0 - 0 - 0"
        Me.sbCoord.Width = &H33
        Me.mainMenu.MenuItems.AddRange(New MenuItem() {Me.miFile, Me.miView, Me.miHelp})
        Me.miFile.Index = 0
        Me.miFile.MenuItems.AddRange(New MenuItem() {Me.miOpenFile, Me.miSaveFile, Me.miSep, Me.miExit})
        Me.miFile.Text = "&File"
        Me.miOpenFile.Index = 0
        Me.miOpenFile.Text = "&Open File..."
        AddHandler Me.miOpenFile.Click, New System.EventHandler(AddressOf Me.miOpenFile_Click)
        Me.miSaveFile.Enabled = False
        Me.miSaveFile.Index = 1
        Me.miSaveFile.Text = "Save File As..."
        AddHandler Me.miSaveFile.Click, New System.EventHandler(AddressOf Me.miSaveFile_Click)
        Me.miSep.Index = 2
        Me.miSep.Text = "-"
        Me.miExit.Index = 3
        Me.miExit.Text = "&Close"
        AddHandler Me.miExit.Click, New System.EventHandler(AddressOf Me.miExit_Click)
        Me.miView.Index = 1
        Me.miView.MenuItems.AddRange(New MenuItem() {Me.miScale})
        Me.miView.Text = "&View"
        Me.miScale.Enabled = False
        Me.miScale.Index = 0
        Me.miScale.MenuItems.AddRange(New MenuItem() {Me.miZoom10, Me.miZoom25, Me.miZoom50, Me.miZoom100, Me.miZoom200, Me.miZoom400, Me.miZoom800})
        Me.miScale.Text = "&Scale"
        Me.miZoom10.Index = 0
        Me.miZoom10.Text = "10%"
        AddHandler Me.miZoom10.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom25.Index = 1
        Me.miZoom25.Text = "25%"
        AddHandler Me.miZoom25.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom50.Index = 2
        Me.miZoom50.Text = "50%"
        AddHandler Me.miZoom50.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom100.Index = 3
        Me.miZoom100.Text = "100%"
        AddHandler Me.miZoom100.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom200.Index = 4
        Me.miZoom200.Text = "200%"
        AddHandler Me.miZoom200.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom400.Index = 5
        Me.miZoom400.Text = "400%"
        AddHandler Me.miZoom400.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miZoom800.Index = 6
        Me.miZoom800.Text = "800%"
        AddHandler Me.miZoom800.Click, New System.EventHandler(AddressOf Me.miZoom_Click)
        Me.miHelp.Index = 2
        Me.miHelp.MenuItems.AddRange(New MenuItem() {Me.miRegister, Me.miFloatLic, Me.miAbout})
        Me.miHelp.Text = "&?"
        Me.miRegister.Index = 0
        Me.miRegister.Text = "Register"
        AddHandler Me.miRegister.Click, New System.EventHandler(AddressOf Me.miRegister_Click)
        Me.miFloatLic.Index = 1
        Me.miFloatLic.Text = "Floating License Registration..."
        AddHandler Me.miFloatLic.Click, New System.EventHandler(AddressOf Me.miFloatLic_Click)
        Me.miAbout.Index = 2
        Me.miAbout.Text = "&About"
        AddHandler Me.miAbout.Click, New System.EventHandler(AddressOf Me.miAbout_Click)
        Me.pnlLayouts.BorderStyle = BorderStyle.FixedSingle
        Me.pnlLayouts.Controls.Add(Me.btnSave)
        Me.pnlLayouts.Controls.Add(Me.btnLoad)
        Me.pnlLayouts.Dock = DockStyle.Top
        Me.pnlLayouts.Location = New Point(0, 0)
        Me.pnlLayouts.Name = "pnlLayouts"
        Me.pnlLayouts.Size = New Size(&H2AF, &H20)
        Me.pnlLayouts.TabIndex = 11
        Me.btnSave.Enabled = False
        Me.btnSave.FlatStyle = FlatStyle.System
        Me.btnSave.Location = New Point(&H60, 4)
        Me.btnSave.Name = "btnSave"
        Me.btnSave.TabIndex = 1
        Me.btnSave.Text = "Save"
        AddHandler Me.btnSave.Click, New System.EventHandler(AddressOf Me.miSaveFile_Click)
        Me.btnLoad.FlatStyle = FlatStyle.System
        Me.btnLoad.Location = New Point(9, 3)
        Me.btnLoad.Name = "btnLoad"
        Me.btnLoad.TabIndex = 0
        Me.btnLoad.Text = "Load"
        AddHandler Me.btnLoad.Click, New System.EventHandler(AddressOf Me.miOpenFile_Click)
        Me.cadPictBox.BackColor = Color.White
        Me.cadPictBox.BorderStyle = BorderStyle.FixedSingle
        Me.cadPictBox.Cursor = Cursors.Default
        Me.cadPictBox.Dock = DockStyle.Fill
        Me.cadPictBox.DoubleBuffering = True
        Me.cadPictBox.Location = New Point(0, &H20)
        Me.cadPictBox.Name = "cadPictBox"
        Me.cadPictBox.Position = New Point(0, 0)
        Me.cadPictBox.Size = New Size(&H2AF, &H10C)
        Me.cadPictBox.TabIndex = 8
        Me.cadPictBox.TabStop = False
        Me.cadPictBox.VirtualSize = New Size(0, 0)
        AddHandler Me.cadPictBox.MouseUp, New MouseEventHandler(AddressOf Me.cadPictBox_MouseUp)
        AddHandler Me.cadPictBox.Paint, New PaintEventHandler(AddressOf Me.cadPictBox_Paint)
        AddHandler Me.cadPictBox.MouseMove, New MouseEventHandler(AddressOf Me.cadPictBox_MouseMove)
        AddHandler Me.cadPictBox.MouseDown, New MouseEventHandler(AddressOf Me.cadPictBox_MouseDown)
        Me.dlgSaveEnt.DefaultExt = "*.xml"
        Me.dlgSaveEnt.Filter = "*.xml|*.xml|*.txt|*.txt"
        Me.AutoScaleBaseSize = New Size(5, 13)
        Me.BackColor = SystemColors.Control
        MyBase.ClientSize = New Size(&H2AF, &H13C)
        MyBase.Controls.Add(Me.cadPictBox)
        MyBase.Controls.Add(Me.pnlLayouts)
        MyBase.Controls.Add(Me.stBar)
        MyBase.Icon = DirectCast(resources.GetObject("$this.Icon"), Icon)
        MyBase.Menu = Me.mainMenu
        MyBase.Name = "MainForm"
        Me.RightToLeft = RightToLeft.No
        MyBase.StartPosition = FormStartPosition.CenterScreen
        Me.Text = "CADImport.NET - Import"
        MyBase.WindowState = FormWindowState.Maximized
        AddHandler MyBase.Closing, New CancelEventHandler(AddressOf Me.MainForm_Closing)
        AddHandler MyBase.Load, New System.EventHandler(AddressOf Me.MainForm_Load)
        AddHandler MyBase.Deactivate, New System.EventHandler(AddressOf Me.MainForm_Deactivate)
        Me.sbOpen.EndInit()
        Me.sbScale.EndInit()
        Me.sbCoord.EndInit()
        Me.pnlLayouts.ResumeLayout(False)
        MyBase.ResumeLayout(False)
    End Sub
#End Region

    Private Sub InitParams()
        Me.aboutFrm = New AboutForm
        Me.xmlImporter = New XMLImporter
        Me.txtImporter = New TXTImporter
        Me.floatLicFrm = New FloatLicForm
        Me.regFrm = New RegForm
    End Sub

    Private Sub InvalidateImage(ByVal sender As Object, ByVal e As EventArgs)
        Me.cadPictBox.Invalidate()
    End Sub

    ''' <summary>
    ''' Loads a CAD file.
    ''' </summary>
    Public Sub LoadFile(ByVal fileName As String)
        If (Not fileName Is Nothing) Then
            If (Not Me.cadImage Is Nothing) Then
                Me.cadImage.Dispose()
                Me.cadImage = Nothing
            End If
            Me.Cursor = Cursors.WaitCursor
            Me.stBar.Panels.Item(0).Text = ApplicationConstants.loadfilestr
            Me.ImageScale = 1.0!
            Me.ImagePreviousScale = 1.0!
            Me.Position = New PointF
            Try
                Me.cadImage = cadImage.CreateImageByExtension(fileName)
            Catch ex As Exception
                MessageBox.Show(String.Format("[{0}] {1}", ex.Message, ex.StackTrace), ApplicationConstants.msgBoxDebugCaption, MessageBoxButtons.OK)
            End Try
            If TypeOf Me.cadImage Is CADRasterImage Then
                CType(Me.cadImage, CADRasterImage).Control = Me.cadPictBox
            End If
        End If
        If (Not Me.cadImage Is Nothing) Then
            Try
                If CADConst.IsWebPath(fileName) Then
                    Me.cadImage.LoadFromWeb(fileName)
                Else
                    Me.cadImage.LoadFromFile(fileName)
                End If
            Catch ex As Exception
                MessageBox.Show(String.Format("[{0}] {1}", ex.Message, ex.StackTrace), ApplicationConstants.msgBoxDebugCaption, MessageBoxButtons.OK)
            End Try
        End If
        Me.stBar.Panels.Item(0).Text = (Path.GetFileName(fileName) & ApplicationConstants.sepstr)
        cadImage.LastLoadedFilePath = Path.GetDirectoryName(fileName)
        Me.SetCADImageOptions()
    End Sub

    Private Sub MainForm_Closing(ByVal sender As Object, ByVal e As CancelEventArgs)
        If (Protection.LicenseType = LicenseType.Floating) Then
            Me.floatLicFrm.DisableFloating()
        Else
            Protection.CloseApplication()
        End If
    End Sub

    Private Sub MainForm_Deactivate(ByVal sender As Object, ByVal e As EventArgs)
        Me.detRMouseDown = False
    End Sub

    Private Sub MainForm_Load(ByVal sender As Object, ByVal e As EventArgs)
        Me.VisibleAreaSize = New SizeF(Me.cadPictBox.Size.Width, Me.cadPictBox.Size.Height)
        If (Protection.LicenseType = LicenseType.Floating) Then
            Me.floatLicFrm.FloatingLicRegister()
        Else
            Protection.Register()
        End If
        CADConst.UseSHXFonts = False
        CADConst.UseMultyTTFFonts = True
        CADConst.UseTTFFonts = True
    End Sub

    Private Sub miAbout_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.aboutFrm.ShowDialog()
    End Sub

    Private Sub miExit_Click(ByVal sender As Object, ByVal e As EventArgs)
        MyBase.Close()
    End Sub

    Private Sub miFloatLic_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.floatLicFrm.ShowDialog()
    End Sub

    Private Sub miOpenFile_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.ShowLoadFileDialog()
        Me.ChangeControlState()
    End Sub

    Private Sub miRegister_Click(ByVal sender As Object, ByVal e As EventArgs)
        Me.regFrm.ShowDialog()
    End Sub

    Private Sub miSaveFile_Click(ByVal sender As Object, ByVal e As EventArgs)
        Dim name As String = Me.cadImage.Converter.Source
        If (Not name Is Nothing) Then
            name = Path.GetFileName(name).Replace(Path.GetExtension(name), String.Empty)
            Me.dlgSaveEnt.FileName = name
        End If
        If ((Me.dlgSaveEnt.ShowDialog = DialogResult.OK) AndAlso (Not Me.dlgSaveEnt.FileName Is Nothing)) Then
            If (Me.dlgSaveEnt.FilterIndex = 1) Then
                Me.xmlImporter.CreateXMLRecords(Me.dlgSaveEnt.FileName, (name & ApplicationConstants.xmlextstr2))
            Else
                Me.txtImporter.CreateTXTRecords((name & ApplicationConstants.txtextstr))
            End If
        End If
    End Sub

    Private Sub miZoom_Click(ByVal sender As Object, ByVal e As EventArgs)
        If (Not Me.cadImage Is Nothing) Then
            Dim i As Single = Me.ImageScale
            Select Case CType(sender, MenuItem).Index
                Case 0
                    i = 0.1!
                    Exit Select
                Case 1
                    i = 0.25!
                    Exit Select
                Case 2
                    i = 0.5!
                    Exit Select
                Case 3
                    i = 1.0!
                    Exit Select
                Case 4
                    i = 2.0!
                    Exit Select
                Case 5
                    i = 4.0!
                    Exit Select
                Case 6
                    i = 8.0!
                    Exit Select
            End Select
            Me.ResetScaling()
            Me.PreviousPosition = New PointF((CSng(Me.cadPictBox.ClientRectangle.Width) / 2.0!), (CSng(Me.cadPictBox.ClientRectangle.Height) / 2.0!))
            Me.ImageScale = i
            Me.cadPictBox.Invalidate()
            Me.SetPictureBoxPosition(Me.Position)
        End If
    End Sub

    Private Sub MoveToPosition(ByRef point As DPoint, ByVal aPos As DPoint, ByVal direction As Integer)
        point.X = (point.X + (direction * aPos.X))
        point.Y = (point.Y + (direction * aPos.Y))
        point.Z = (point.Z + (direction * aPos.Z))
    End Sub

    Private Sub NotSupportedInCurrentVersion()
        Me.Cursor = Cursors.Default
        Me.cadPictBox.Visible = True
        Me.stBar.Panels.Item(0).Text = ApplicationConstants.notsupportedstr
        MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr)
    End Sub

    ''' <summary>
    ''' Sets the scale and position of the current CAD image to its original values.
    ''' </summary>
    Public Sub ResetScaling()
        Me.ImageScale = 1.0!
        Me.ImagePreviousScale = 1.0!
        Me.LeftImagePosition = ((Me.cadPictBox.ClientRectangle.Width - Me.VisibleAreaSize.Width) / 2.0!)
        Me.TopImagePosition = ((Me.cadPictBox.ClientRectangle.Height - Me.VisibleAreaSize.Height) / 2.0!)
        Me.cadPictBox.Invalidate()
    End Sub

    Public Sub SetCADImageOptions()
        Me.cadImage.UseWinEllipse = False
        Me.cadImage.IsShowLineWeight = False
        Me.cadImage.UseDoubleBuffering = True
        Me.Cursor = Cursors.Default
        ObjEntity.cadImage = Me.cadImage
        Me.xmlImporter.Image = Me.cadImage
        Me.txtImporter.Image = Me.cadImage
        Me.DoResize()
        Me.SetPictureBoxPosition(Me.Position)
    End Sub

    Private Sub SetPictureBoxPosition(ByVal value As PointF)
        Dim w1 As Integer
        Dim h1 As Integer
        If (value.X > 0.0!) Then
            w1 = 0
        Else
            w1 = CInt(Math.Abs(value.X))
        End If
        If (w1 > Me.cadPictBox.VirtualSize.Width) Then
            w1 = Me.cadPictBox.VirtualSize.Width
        End If
        If (value.Y > 0.0!) Then
            h1 = 0
        Else
            h1 = CInt(Math.Abs(value.Y))
        End If
        If (h1 > Me.cadPictBox.VirtualSize.Height) Then
            h1 = Me.cadPictBox.VirtualSize.Height
        End If
        Me.cadPictBox.SetPositionNoInvalidate(New Point(w1, h1))
    End Sub

    Public Sub SetSizePictureBox(ByVal sz As Size)
        Me.cadPictBox.SetVirtualSizeNoInvalidate(sz)
    End Sub

    Private Sub Shift()
        Me.LeftImagePosition = (Me.PreviousPosition.X - (((Me.PreviousPosition.X - Me.LeftImagePosition) * Me.ImageScale) / Me.ImagePreviousScale))
        Me.TopImagePosition = (Me.PreviousPosition.Y - (((Me.PreviousPosition.Y - Me.TopImagePosition) * Me.ImageScale) / Me.ImagePreviousScale))
        Me.ImagePreviousScale = Me.ImageScale
    End Sub

    Public Sub ShowLoadFileDialog()
        Me.cadPictBox.Enabled = False
        If (Me.dlgOpenFile.ShowDialog <> DialogResult.OK) Then
            Me.cadPictBox.Visible = True
            Me.cadPictBox.Enabled = True
        Else
            Me.cadPictBox.Enabled = True
            Me.LoadFile(Me.dlgOpenFile.FileName)
        End If
    End Sub

    Private Sub Zoom(ByVal i As Single)
        If (Not Me.cadImage Is Nothing) Then
            Me.sc = (Me.sc * i)
            If (Me.sc < 0.005!) Then
                Me.sc = 0.005!
            End If
            Me.cadPictBox.Invalidate()
        End If
    End Sub


    ' Properties
    ''' <summary>
    ''' An X coordinate of the last mouse click position on the screen.
    ''' </summary>
    Public ReadOnly Property CurrentXClickPosition() As Integer
        Get
            Return Me.cX
        End Get
    End Property

    ''' <summary>
    ''' A Y coordinate of the last mouse click position on the screen.
    ''' </summary>
    Public ReadOnly Property CurrentYClickPosition() As Integer
        Get
            Return Me.cY
        End Get
    End Property

    ''' <summary>
    ''' A CAD image displayed in the main application form.
    ''' </summary>
    Public Property Image() As cadImage
        Get
            Return Me.cadImage
        End Get
        Set(ByVal value As cadImage)
            Me.cadImage = value
        End Set
    End Property

    Public ReadOnly Property ImageDRect() As DRect
        Get
            Return New DRect(CDbl(Me.LeftImagePosition), CDbl(Me.TopImagePosition), CDbl((Me.VisibleAreaSize.Width * Me.ImageScale)), CDbl((Me.VisibleAreaSize.Height * Me.ImageScale)))
        End Get
    End Property

    ''' <summary>
    ''' A previous value of the scale in which the current CAD image is displayed in the viewer.
    ''' </summary>
    Public Property ImagePreviousScale() As Single
        Get
            Return Me.prev_sc
        End Get
        Set(ByVal value As Single)
            Me.prev_sc = value
        End Set
    End Property

    Public ReadOnly Property ImageRectangleF() As RectangleF
        Get
            Return New RectangleF(Me.LeftImagePosition, Me.TopImagePosition, (Me.VisibleAreaSize.Width * Me.ImageScale), (Me.VisibleAreaSize.Height * Me.ImageScale))
        End Get
    End Property

    ''' <summary>
    ''' A scale in which the current CAD image is displayed in the viewer.
    ''' </summary>
    Public Property ImageScale() As Single
        Get
            Return Me.sc
        End Get
        Set(ByVal value As Single)
            Me.sc = value
        End Set
    End Property

    Public Property LeftImagePosition() As Single
        Get
            Return Me.pos.X
        End Get
        Set(ByVal value As Single)
            Me.pos.X = value
        End Set
    End Property

    ''' <summary>
    ''' A position of the current CAD image in relation to the top-left corner of the viewer.
    ''' </summary>
    Public Property Position() As PointF
        Get
            Return Me.pos
        End Get
        Set(ByVal value As PointF)
            Me.pos = value
        End Set
    End Property

    ''' <summary>
    ''' A previous position of the current CAD image in relation to the top-left corner of the viewer.
    ''' </summary>
    Public Property PreviousPosition() As PointF
        Get
            Return Me.old_Pos
        End Get
        Set(ByVal value As PointF)
            Me.old_Pos = value
        End Set
    End Property

    Private ReadOnly Property RealScale() As String
        Get
            If (Not Me.cadImage Is Nothing) Then
                Return String.Format("{0,2:F}%", (((CDbl((Me.VisibleAreaSize.Width * Me.ImageScale)) / Me.cadImage.AbsWidth) * Me.cadImage.MMToPixelX) * 100))
            End If
            Return String.Format("{0}%", Me.ImageScale)
        End Get
    End Property

    Public Property TopImagePosition() As Single
        Get
            Return Me.pos.Y
        End Get
        Set(ByVal value As Single)
            Me.pos.Y = value
        End Set
    End Property

    Public Property VisibleAreaSize() As SizeF
        Get
            Return Me.visibleArea
        End Get
        Set(ByVal value As SizeF)
            Me.visibleArea = value
        End Set
    End Property


    ' Fields
    Private aboutFrm As AboutForm
    Private btnLoad As Button
    Private btnSave As Button
    Private cadImage As cadImage
    Private cadPictBox As CADPictureBox
    Private cX As Integer
    Private cY As Integer
    Private detRMouseDown As Boolean
    Private dlgOpenFile As OpenFileDialog
    Private dlgSaveEnt As SaveFileDialog
    Private dlgSaveImg As SaveFileDialog
    Private floatLicFrm As FloatLicForm
    Private mainMenu As mainMenu
    Private miAbout As MenuItem
    Private miExit As MenuItem
    Private miFile As MenuItem
    Private miFloatLic As MenuItem
    Private miHelp As MenuItem
    Private miOpenFile As MenuItem
    Private miRegister As MenuItem
    Private miSaveFile As MenuItem
    Private miScale As MenuItem
    Private miSep As MenuItem
    Private miView As MenuItem
    Private miZoom10 As MenuItem
    Private miZoom100 As MenuItem
    Private miZoom200 As MenuItem
    Private miZoom25 As MenuItem
    Private miZoom400 As MenuItem
    Private miZoom50 As MenuItem
    Private miZoom800 As MenuItem
    Private old_Pos As PointF
    Private pnlLayouts As Panel
    Private pos As PointF
    Private prev_sc As Single
    Private regFrm As RegForm
    Private sbCoord As StatusBarPanel
    Private sbOpen As StatusBarPanel
    Private sbScale As StatusBarPanel
    Private sc As Single
    Private stBar As StatusBar
    Private txtImporter As txtImporter
    Private visibleArea As SizeF
    Private xmlImporter As xmlImporter
End Class

Friend MustInherit Class ApplicationConstants
    ' Methods
    Protected Sub New()
    End Sub

    Public Shared Function GetColor(ByVal val As Color) As String
        If val.Equals(CADConst.clNone) Then
            Return "[black/white]"
        End If
        If val.Equals(CADConst.clByBlock) Then
            Return "[ByBlock]"
        End If
        If val.Equals(CADConst.clByLayer) Then
            Return "[ByLayer]"
        End If
        Return val.ToString
    End Function


    ' Fields
    Public Shared ReadOnly appnamestr As String = "CADImportNet Demo"
    Public Shared ReadOnly hatchStyle As String() = New String() {"BDiagonal", "Cross", "DiagCross", "FDiagonal", "Horizontal", "PatternData", "Solid", "Vertical"}
    Public Shared ReadOnly loadfilestr As String = "Loading file..."
    Public Shared ReadOnly msgBoxDebugCaption As String = "Debug application message"
    Public Shared ReadOnly notsupportedextstr As String = "Not supported in current version!"
    Public Shared ReadOnly notsupportedstr As String = "not supported"
    Public Shared ReadOnly sepstr As String = " - "
    Public Shared ReadOnly sepstr2 As String = " : "
    Public Shared ReadOnly sepstr3 As String = (String.Empty & ChrW(13) & ChrW(10))
    Public Shared ReadOnly txtextstr As String = ".txt"
    Public Shared ReadOnly xmlextstr As String = ".XML"
    Public Shared ReadOnly xmlextstr2 As String = ".xml"
End Class




