//To compile the project you need the extended version of the CADImport .NET library
//that allows to access CAD entities directly.  You can get this version of the 
//library free of charge by emailing your request to: info@cadsofttools.com
using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Globalization;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using CADImport;
using CADImport.RasterImage;
using CADImport.FaceModule;
using CADImport.CADImportForms;
#region HPGL
#if HPGL
using CADImport.HPGL2;
#endif
#endregion

namespace CADImportNetDemos.CADMergeFilesDemo
{
	#region Help
	/// <summary>
	/// Represents the main application form in which CAD images can be viewed and merged.
	/// </summary>
	#endregion Help
	public class MainForm: System.Windows.Forms.Form
	{	
		private CADImportNetDemos.CADMergeFilesDemo.MergeForm mergeForm;
		private CADImport.CADImportForms.AboutForm aboutFrm;
		private CADImport.CADImportForms.LayerForm lForm;
		private CADImport.CADImportForms.SHXForm shxFrm;		
		private CADImport.Printing.PrintingForm prtForm;		
		private CADImport.CADImportForms.OpenFileFromWebForm openFromWebForm;
		private CADImport.CADImportForms.SetRasterSizeForm rasterSizeForm;		
		#region protect
#if protect
		private RegForm regFrm;
#endif
#if (protect && floatprotect)
		private FloatLicForm floatLicFrm;
#endif
		#endregion protect		
								
		private PointF old_Pos;						
		private PointF pos;		
		private System.Drawing.SizeF visibleArea;		
		private float sc;
		private float prev_sc;		
		private int cX, cY;		
		private bool detRMouseDown;		
		private bool dimVisible;
		private bool drawMode;
		private bool showLineWeight;
		private bool textVisible;
		private Orbit3D Orb3D;				
		private int curLngIndex;		
		private CADImport.CADImage cadImage;		
		private CADImport.FaceModule.ClipRect clipRectangle;				
		private CADImport.FaceModule.CADPictureBox cadPictBox;
		private CADImport.FaceModule.ListItemSelectEventHandler listItemSelectDel;		
		
		private static string openFileName;				
		private static readonly string fileSettingsName;																		

		private System.Windows.Forms.StatusBar stBar;
		internal System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem mExit;
		private System.Windows.Forms.MenuItem mOpenFile;
		private System.Windows.Forms.MenuItem mSaveFile;
		private System.Windows.Forms.MenuItem mFile;
		private System.Windows.Forms.MenuItem aboutMenuItem;
		private System.Windows.Forms.StatusBarPanel sbOpen;
		private System.Windows.Forms.StatusBarPanel sbScale;
		private System.Windows.Forms.ToolBarButton tlbOpen;
		private System.Windows.Forms.ToolBarButton tlbZoomIn;
		private System.Windows.Forms.ToolBarButton tlbZoomOut;
		private System.Windows.Forms.ToolBarButton tlbSave;
		private System.Windows.Forms.ToolBarButton tlbWhite;
		private System.Windows.Forms.ToolBarButton tlbBlack;
		private System.Windows.Forms.ToolBarButton tlbColor;
		private System.Windows.Forms.ToolBar tlbTool;
		private System.Windows.Forms.ToolBarButton tlbOSX;
		private System.Windows.Forms.ToolBarButton tlb3d;
		private System.Windows.Forms.ToolBarButton tlbBlackMode;
		private System.Windows.Forms.ToolBarButton tlbColorMode;
		private System.Windows.Forms.ContextMenu orbitViewMenu;
		private System.Windows.Forms.MenuItem InitialViewMenuItem;
		private System.Windows.Forms.MenuItem TopViewMenuItem;
		private System.Windows.Forms.MenuItem BottomViewMenuItem;
		private System.Windows.Forms.MenuItem LeftViewMenuItem;
		private System.Windows.Forms.MenuItem RightViewMenuItem;
		private System.Windows.Forms.MenuItem FrontViewMenuItem;
		private System.Windows.Forms.MenuItem BackViewMenuItem;
		private System.Windows.Forms.MenuItem SWViewMenuItem;
		private System.Windows.Forms.MenuItem SEViewMenuItem;
		private System.Windows.Forms.MenuItem NEViewMenuItem;
		private System.Windows.Forms.MenuItem NWViewMenuItem;
		private System.Windows.Forms.StatusBarPanel sbCoord;
		private System.Windows.Forms.ToolBarButton tlbDefSize;
		private System.Windows.Forms.ToolBarButton tlbTextShow;
		private System.Windows.Forms.ToolBarButton tlbDimShow;
		private System.Windows.Forms.ToolBarButton tlbSaveImageClip;
		private System.Windows.Forms.ToolBarButton tlbPrint;
		private System.Windows.Forms.MenuItem menuItemReg;
		private System.Windows.Forms.MenuItem closeMenuItem;
		private System.Windows.Forms.MenuItem printMenuItem;
		private System.Windows.Forms.MenuItem helpMenuItem;
		private System.Windows.Forms.MenuItem printprevItem;		
		private System.Windows.Forms.MenuItem print2Item;
		private System.Windows.Forms.ToolBarButton tlbMergeFiles;
		private System.Windows.Forms.MenuItem mnSaveAsDXF;		
		private System.Windows.Forms.ToolBarButton tlbRect;
		private System.Windows.Forms.SaveFileDialog dlgSaveImg;
		private System.Windows.Forms.ColorDialog dlgSetColor;
		private System.Windows.Forms.PrintPreviewDialog dlgPrintPrev;
		private System.Windows.Forms.OpenFileDialog dlgOpenFile;
		private System.Windows.Forms.SaveFileDialog dlgSaveDXF;
		private System.Windows.Forms.MenuItem miFloatLic;
		private System.Windows.Forms.PrintPreviewDialog printPrevDlg;
		private System.Windows.Forms.MenuItem miMerge;
		private System.Windows.Forms.ImageList imgLstToolBtn;
		private System.ComponentModel.IContainer components;				

		#region Help
		/// <summary>
		/// A CAD image displayed in the main application form.
		/// </summary>
		#endregion Help		
		public CADImage Image
		{
			get
			{
				return cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets or sets lines weight visibility
		/// </summary>
#endif	
		#endregion
		public bool ShowLineWeight
		{
			get
			{
				return this.showLineWeight;
			}
			set
			{
				this.showLineWeight = value;				
			}
		}

		#region Help
		/// <summary>
		/// An X coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentXClickPosition
		{
			get
			{
				return this.cX;
			}
		}
		#region Help
		/// <summary>
		/// A Y coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentYClickPosition
		{
			get
			{
				return this.cY;
			}
		}		

		#region Help
		/// <summary>
		/// A previous position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>
		#endregion Help
		public PointF PreviousPosition
		{
			get
			{
				return this.old_Pos;
			}
			set
			{
				this.old_Pos = value;
			}
		}			
	
		#region Help
#if RUSHELP
		/// <summary>
		///      
		/// </summary>
#else		
		/// <summary>
		/// A position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>		
#endif
		#endregion
		public PointF Position
		{
			get
			{
				return this.pos;
			}
			set
			{
				this.pos = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         X
		/// </summary>
#else		
		/// <summary>
		/// Gets or sets a position of the drawing image by X axis
		/// </summary>
	
#endif
		#endregion
		public float LeftImagePosition
		{
			get
			{
				return this.pos.X;
			}
			set
			{
				this.pos.X = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         Y
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a position of the drawing image by Y axis
		/// </summary>
#endif
		#endregion
		public float TopImagePosition
		{
			get
			{
				return this.pos.Y;
			}
			set
			{
				this.pos.Y = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
		/// <summary>
		/// Gets or sets base dimensions of the image drawing area
		/// </summary>
#endif
		#endregion
		public SizeF VisibleAreaSize
		{
			get
			{
				return this.visibleArea;
			}
			set
			{
				this.visibleArea = value;
			}
		}

		#region Help
		/// <summary>
		/// A scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImageScale
		{
			get
			{
				return this.sc;
			}
			set
			{
				this.sc = value;
			}
		}

		#region Help
		/// <summary>
		/// A previous value of the scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImagePreviousScale
		{
			get
			{
				return this.prev_sc;
			}
			set
			{
				this.prev_sc = value;
			}
		}

		#region Help
#if	RUSHELP
		/// <summary>
		///       - 
		/// (    true - )
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a flag of the drawing mode
		/// (if <b>true</b> - in color, else - in black-and-white)
		/// </summary>
#endif
		#endregion
		public bool DrawMode
		{
			get
			{
				return this.drawMode;
			}
			set
			{				
				this.drawMode = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public RectangleF ImageRectangleF
		{
			get
			{
				return new RectangleF(this.LeftImagePosition, this.TopImagePosition, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public DRect ImageDRect
		{
			get
			{
				return new DRect(this.LeftImagePosition, this.TopImagePosition, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		private string RealScale
		{
			get
			{
				if(cadImage != null)
					return string.Format("{0,2:F}%", (((this.VisibleAreaSize.Width * ImageScale) / this.cadImage.AbsWidth) * cadImage.MMToPixelX) * 100.0);
				else 
					return string.Format("{0}%", ImageScale);
			}
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.dlgSaveImg = new System.Windows.Forms.SaveFileDialog();
			this.stBar = new System.Windows.Forms.StatusBar();
			this.sbOpen = new System.Windows.Forms.StatusBarPanel();
			this.sbScale = new System.Windows.Forms.StatusBarPanel();
			this.sbCoord = new System.Windows.Forms.StatusBarPanel();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.mFile = new System.Windows.Forms.MenuItem();
			this.mOpenFile = new System.Windows.Forms.MenuItem();
			this.miMerge = new System.Windows.Forms.MenuItem();
			this.mSaveFile = new System.Windows.Forms.MenuItem();
			this.mnSaveAsDXF = new System.Windows.Forms.MenuItem();
			this.printMenuItem = new System.Windows.Forms.MenuItem();
			this.print2Item = new System.Windows.Forms.MenuItem();
			this.printprevItem = new System.Windows.Forms.MenuItem();
			this.closeMenuItem = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.mExit = new System.Windows.Forms.MenuItem();
			this.helpMenuItem = new System.Windows.Forms.MenuItem();
			this.menuItemReg = new System.Windows.Forms.MenuItem();
			this.miFloatLic = new System.Windows.Forms.MenuItem();
			this.aboutMenuItem = new System.Windows.Forms.MenuItem();
			this.dlgSetColor = new System.Windows.Forms.ColorDialog();
			this.tlbTool = new System.Windows.Forms.ToolBar();
			this.tlbOpen = new System.Windows.Forms.ToolBarButton();
			this.tlbMergeFiles = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomIn = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomOut = new System.Windows.Forms.ToolBarButton();
			this.tlbSave = new System.Windows.Forms.ToolBarButton();
			this.tlbWhite = new System.Windows.Forms.ToolBarButton();
			this.tlbBlack = new System.Windows.Forms.ToolBarButton();
			this.tlbColor = new System.Windows.Forms.ToolBarButton();
			this.tlb3d = new System.Windows.Forms.ToolBarButton();
			this.orbitViewMenu = new System.Windows.Forms.ContextMenu();
			this.InitialViewMenuItem = new System.Windows.Forms.MenuItem();
			this.TopViewMenuItem = new System.Windows.Forms.MenuItem();
			this.BottomViewMenuItem = new System.Windows.Forms.MenuItem();
			this.LeftViewMenuItem = new System.Windows.Forms.MenuItem();
			this.RightViewMenuItem = new System.Windows.Forms.MenuItem();
			this.FrontViewMenuItem = new System.Windows.Forms.MenuItem();
			this.BackViewMenuItem = new System.Windows.Forms.MenuItem();
			this.SWViewMenuItem = new System.Windows.Forms.MenuItem();
			this.SEViewMenuItem = new System.Windows.Forms.MenuItem();
			this.NEViewMenuItem = new System.Windows.Forms.MenuItem();
			this.NWViewMenuItem = new System.Windows.Forms.MenuItem();
			this.tlbOSX = new System.Windows.Forms.ToolBarButton();
			this.tlbBlackMode = new System.Windows.Forms.ToolBarButton();
			this.tlbColorMode = new System.Windows.Forms.ToolBarButton();
			this.tlbDefSize = new System.Windows.Forms.ToolBarButton();
			this.tlbTextShow = new System.Windows.Forms.ToolBarButton();
			this.tlbDimShow = new System.Windows.Forms.ToolBarButton();
			this.tlbSaveImageClip = new System.Windows.Forms.ToolBarButton();
			this.tlbPrint = new System.Windows.Forms.ToolBarButton();
			this.tlbRect = new System.Windows.Forms.ToolBarButton();
			this.cadPictBox = new CADImport.FaceModule.CADPictureBox();
			this.printPrevDlg = new System.Windows.Forms.PrintPreviewDialog();
			this.dlgOpenFile = new System.Windows.Forms.OpenFileDialog();
			this.dlgSaveDXF = new System.Windows.Forms.SaveFileDialog();
			this.imgLstToolBtn = new System.Windows.Forms.ImageList(this.components);
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).BeginInit();
			this.SuspendLayout();
			// 
			// dlgSaveImg
			// 
			this.dlgSaveImg.Filter = resources.GetString("dlgSaveImg.Filter");
			this.dlgSaveImg.RestoreDirectory = true;
			this.dlgSaveImg.Title = resources.GetString("dlgSaveImg.Title");
			// 
			// stBar
			// 
			this.stBar.AccessibleDescription = resources.GetString("stBar.AccessibleDescription");
			this.stBar.AccessibleName = resources.GetString("stBar.AccessibleName");
			this.stBar.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("stBar.Anchor")));
			this.stBar.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("stBar.BackgroundImage")));
			this.stBar.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("stBar.Dock")));
			this.stBar.Enabled = ((bool)(resources.GetObject("stBar.Enabled")));
			this.stBar.Font = ((System.Drawing.Font)(resources.GetObject("stBar.Font")));
			this.stBar.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("stBar.ImeMode")));
			this.stBar.Location = ((System.Drawing.Point)(resources.GetObject("stBar.Location")));
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.sbOpen,
																					 this.sbScale,
																					 this.sbCoord});
			this.stBar.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("stBar.RightToLeft")));
			this.stBar.ShowPanels = true;
			this.stBar.Size = ((System.Drawing.Size)(resources.GetObject("stBar.Size")));
			this.stBar.TabIndex = ((int)(resources.GetObject("stBar.TabIndex")));
			this.stBar.Text = resources.GetString("stBar.Text");
			this.stBar.Visible = ((bool)(resources.GetObject("stBar.Visible")));
			// 
			// sbOpen
			// 
			this.sbOpen.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("sbOpen.Alignment")));
			this.sbOpen.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbOpen.Icon = ((System.Drawing.Icon)(resources.GetObject("sbOpen.Icon")));
			this.sbOpen.MinWidth = ((int)(resources.GetObject("sbOpen.MinWidth")));
			this.sbOpen.Text = resources.GetString("sbOpen.Text");
			this.sbOpen.ToolTipText = resources.GetString("sbOpen.ToolTipText");
			this.sbOpen.Width = ((int)(resources.GetObject("sbOpen.Width")));
			// 
			// sbScale
			// 
			this.sbScale.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("sbScale.Alignment")));
			this.sbScale.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbScale.Icon = ((System.Drawing.Icon)(resources.GetObject("sbScale.Icon")));
			this.sbScale.MinWidth = ((int)(resources.GetObject("sbScale.MinWidth")));
			this.sbScale.Text = resources.GetString("sbScale.Text");
			this.sbScale.ToolTipText = resources.GetString("sbScale.ToolTipText");
			this.sbScale.Width = ((int)(resources.GetObject("sbScale.Width")));
			// 
			// sbCoord
			// 
			this.sbCoord.Alignment = ((System.Windows.Forms.HorizontalAlignment)(resources.GetObject("sbCoord.Alignment")));
			this.sbCoord.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbCoord.Icon = ((System.Drawing.Icon)(resources.GetObject("sbCoord.Icon")));
			this.sbCoord.MinWidth = ((int)(resources.GetObject("sbCoord.MinWidth")));
			this.sbCoord.Text = resources.GetString("sbCoord.Text");
			this.sbCoord.ToolTipText = resources.GetString("sbCoord.ToolTipText");
			this.sbCoord.Width = ((int)(resources.GetObject("sbCoord.Width")));
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.mFile,
																					 this.helpMenuItem});
			this.mainMenu.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("mainMenu.RightToLeft")));
			// 
			// mFile
			// 
			this.mFile.Enabled = ((bool)(resources.GetObject("mFile.Enabled")));
			this.mFile.Index = 0;
			this.mFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				  this.mOpenFile,
																				  this.miMerge,
																				  this.mSaveFile,
																				  this.mnSaveAsDXF,
																				  this.printMenuItem,
																				  this.print2Item,
																				  this.printprevItem,
																				  this.closeMenuItem,
																				  this.menuItem2,
																				  this.mExit});
			this.mFile.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("mFile.Shortcut")));
			this.mFile.ShowShortcut = ((bool)(resources.GetObject("mFile.ShowShortcut")));
			this.mFile.Text = resources.GetString("mFile.Text");
			this.mFile.Visible = ((bool)(resources.GetObject("mFile.Visible")));
			// 
			// mOpenFile
			// 
			this.mOpenFile.Enabled = ((bool)(resources.GetObject("mOpenFile.Enabled")));
			this.mOpenFile.Index = 0;
			this.mOpenFile.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("mOpenFile.Shortcut")));
			this.mOpenFile.ShowShortcut = ((bool)(resources.GetObject("mOpenFile.ShowShortcut")));
			this.mOpenFile.Text = resources.GetString("mOpenFile.Text");
			this.mOpenFile.Visible = ((bool)(resources.GetObject("mOpenFile.Visible")));
			this.mOpenFile.Click += new System.EventHandler(this.Open_Click);
			// 
			// miMerge
			// 
			this.miMerge.Enabled = ((bool)(resources.GetObject("miMerge.Enabled")));
			this.miMerge.Index = 1;
			this.miMerge.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("miMerge.Shortcut")));
			this.miMerge.ShowShortcut = ((bool)(resources.GetObject("miMerge.ShowShortcut")));
			this.miMerge.Text = resources.GetString("miMerge.Text");
			this.miMerge.Visible = ((bool)(resources.GetObject("miMerge.Visible")));
			this.miMerge.Click += new System.EventHandler(this.miMerge_Click);
			// 
			// mSaveFile
			// 
			this.mSaveFile.Enabled = ((bool)(resources.GetObject("mSaveFile.Enabled")));
			this.mSaveFile.Index = 2;
			this.mSaveFile.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("mSaveFile.Shortcut")));
			this.mSaveFile.ShowShortcut = ((bool)(resources.GetObject("mSaveFile.ShowShortcut")));
			this.mSaveFile.Text = resources.GetString("mSaveFile.Text");
			this.mSaveFile.Visible = ((bool)(resources.GetObject("mSaveFile.Visible")));
			this.mSaveFile.Click += new System.EventHandler(this.ImgSave_Click);
			// 
			// mnSaveAsDXF
			// 
			this.mnSaveAsDXF.Enabled = ((bool)(resources.GetObject("mnSaveAsDXF.Enabled")));
			this.mnSaveAsDXF.Index = 3;
			this.mnSaveAsDXF.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("mnSaveAsDXF.Shortcut")));
			this.mnSaveAsDXF.ShowShortcut = ((bool)(resources.GetObject("mnSaveAsDXF.ShowShortcut")));
			this.mnSaveAsDXF.Text = resources.GetString("mnSaveAsDXF.Text");
			this.mnSaveAsDXF.Visible = ((bool)(resources.GetObject("mnSaveAsDXF.Visible")));
			this.mnSaveAsDXF.Click += new System.EventHandler(this.mnSaveAsDXF_Click);
			// 
			// printMenuItem
			// 
			this.printMenuItem.Enabled = ((bool)(resources.GetObject("printMenuItem.Enabled")));
			this.printMenuItem.Index = 4;
			this.printMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("printMenuItem.Shortcut")));
			this.printMenuItem.ShowShortcut = ((bool)(resources.GetObject("printMenuItem.ShowShortcut")));
			this.printMenuItem.Text = resources.GetString("printMenuItem.Text");
			this.printMenuItem.Visible = ((bool)(resources.GetObject("printMenuItem.Visible")));
			this.printMenuItem.Click += new System.EventHandler(this.printMenuItem_Click);
			// 
			// print2Item
			// 
			this.print2Item.Enabled = ((bool)(resources.GetObject("print2Item.Enabled")));
			this.print2Item.Index = 5;
			this.print2Item.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("print2Item.Shortcut")));
			this.print2Item.ShowShortcut = ((bool)(resources.GetObject("print2Item.ShowShortcut")));
			this.print2Item.Text = resources.GetString("print2Item.Text");
			this.print2Item.Visible = ((bool)(resources.GetObject("print2Item.Visible")));
			this.print2Item.Click += new System.EventHandler(this.btnPrintPrevCust_Click);
			// 
			// printprevItem
			// 
			this.printprevItem.Enabled = ((bool)(resources.GetObject("printprevItem.Enabled")));
			this.printprevItem.Index = 6;
			this.printprevItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("printprevItem.Shortcut")));
			this.printprevItem.ShowShortcut = ((bool)(resources.GetObject("printprevItem.ShowShortcut")));
			this.printprevItem.Text = resources.GetString("printprevItem.Text");
			this.printprevItem.Visible = ((bool)(resources.GetObject("printprevItem.Visible")));
			this.printprevItem.Click += new System.EventHandler(this.printprevItem_Click);
			// 
			// closeMenuItem
			// 
			this.closeMenuItem.Enabled = ((bool)(resources.GetObject("closeMenuItem.Enabled")));
			this.closeMenuItem.Index = 7;
			this.closeMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("closeMenuItem.Shortcut")));
			this.closeMenuItem.ShowShortcut = ((bool)(resources.GetObject("closeMenuItem.ShowShortcut")));
			this.closeMenuItem.Text = resources.GetString("closeMenuItem.Text");
			this.closeMenuItem.Visible = ((bool)(resources.GetObject("closeMenuItem.Visible")));
			this.closeMenuItem.Click += new System.EventHandler(this.closeMenuItem_Click);
			// 
			// menuItem2
			// 
			this.menuItem2.Enabled = ((bool)(resources.GetObject("menuItem2.Enabled")));
			this.menuItem2.Index = 8;
			this.menuItem2.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("menuItem2.Shortcut")));
			this.menuItem2.ShowShortcut = ((bool)(resources.GetObject("menuItem2.ShowShortcut")));
			this.menuItem2.Text = resources.GetString("menuItem2.Text");
			this.menuItem2.Visible = ((bool)(resources.GetObject("menuItem2.Visible")));
			// 
			// mExit
			// 
			this.mExit.Enabled = ((bool)(resources.GetObject("mExit.Enabled")));
			this.mExit.Index = 9;
			this.mExit.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("mExit.Shortcut")));
			this.mExit.ShowShortcut = ((bool)(resources.GetObject("mExit.ShowShortcut")));
			this.mExit.Text = resources.GetString("mExit.Text");
			this.mExit.Visible = ((bool)(resources.GetObject("mExit.Visible")));
			this.mExit.Click += new System.EventHandler(this.menuItem3_Click);
			// 
			// helpMenuItem
			// 
			this.helpMenuItem.Enabled = ((bool)(resources.GetObject("helpMenuItem.Enabled")));
			this.helpMenuItem.Index = 1;
			this.helpMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemReg,
																						 this.miFloatLic,
																						 this.aboutMenuItem});
			this.helpMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("helpMenuItem.Shortcut")));
			this.helpMenuItem.ShowShortcut = ((bool)(resources.GetObject("helpMenuItem.ShowShortcut")));
			this.helpMenuItem.Text = resources.GetString("helpMenuItem.Text");
			this.helpMenuItem.Visible = ((bool)(resources.GetObject("helpMenuItem.Visible")));
			// 
			// menuItemReg
			// 
			this.menuItemReg.Enabled = ((bool)(resources.GetObject("menuItemReg.Enabled")));
			this.menuItemReg.Index = 0;
			this.menuItemReg.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("menuItemReg.Shortcut")));
			this.menuItemReg.ShowShortcut = ((bool)(resources.GetObject("menuItemReg.ShowShortcut")));
			this.menuItemReg.Text = resources.GetString("menuItemReg.Text");
			this.menuItemReg.Visible = ((bool)(resources.GetObject("menuItemReg.Visible")));
			this.menuItemReg.Click += new System.EventHandler(this.toolMenuItem_Click);
			// 
			// miFloatLic
			// 
			this.miFloatLic.Enabled = ((bool)(resources.GetObject("miFloatLic.Enabled")));
			this.miFloatLic.Index = 1;
			this.miFloatLic.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("miFloatLic.Shortcut")));
			this.miFloatLic.ShowShortcut = ((bool)(resources.GetObject("miFloatLic.ShowShortcut")));
			this.miFloatLic.Text = resources.GetString("miFloatLic.Text");
			this.miFloatLic.Visible = ((bool)(resources.GetObject("miFloatLic.Visible")));
			this.miFloatLic.Click += new System.EventHandler(this.miFloatLic_Click);
			// 
			// aboutMenuItem
			// 
			this.aboutMenuItem.Enabled = ((bool)(resources.GetObject("aboutMenuItem.Enabled")));
			this.aboutMenuItem.Index = 2;
			this.aboutMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("aboutMenuItem.Shortcut")));
			this.aboutMenuItem.ShowShortcut = ((bool)(resources.GetObject("aboutMenuItem.ShowShortcut")));
			this.aboutMenuItem.Text = resources.GetString("aboutMenuItem.Text");
			this.aboutMenuItem.Visible = ((bool)(resources.GetObject("aboutMenuItem.Visible")));
			this.aboutMenuItem.Click += new System.EventHandler(this.aboutMenuItem_Click);
			// 
			// tlbTool
			// 
			this.tlbTool.AccessibleDescription = resources.GetString("tlbTool.AccessibleDescription");
			this.tlbTool.AccessibleName = resources.GetString("tlbTool.AccessibleName");
			this.tlbTool.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("tlbTool.Anchor")));
			this.tlbTool.Appearance = ((System.Windows.Forms.ToolBarAppearance)(resources.GetObject("tlbTool.Appearance")));
			this.tlbTool.AutoSize = ((bool)(resources.GetObject("tlbTool.AutoSize")));
			this.tlbTool.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("tlbTool.BackgroundImage")));
			this.tlbTool.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.tlbTool.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.tlbOpen,
																					   this.tlbMergeFiles,
																					   this.tlbZoomIn,
																					   this.tlbZoomOut,
																					   this.tlbSave,
																					   this.tlbWhite,
																					   this.tlbBlack,
																					   this.tlbColor,
																					   this.tlb3d,
																					   this.tlbOSX,
																					   this.tlbBlackMode,
																					   this.tlbColorMode,
																					   this.tlbDefSize,
																					   this.tlbTextShow,
																					   this.tlbDimShow,
																					   this.tlbSaveImageClip,
																					   this.tlbPrint,
																					   this.tlbRect});
			this.tlbTool.ButtonSize = ((System.Drawing.Size)(resources.GetObject("tlbTool.ButtonSize")));
			this.tlbTool.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("tlbTool.Dock")));
			this.tlbTool.DropDownArrows = ((bool)(resources.GetObject("tlbTool.DropDownArrows")));
			this.tlbTool.Enabled = ((bool)(resources.GetObject("tlbTool.Enabled")));
			this.tlbTool.Font = ((System.Drawing.Font)(resources.GetObject("tlbTool.Font")));
			this.tlbTool.ImageList = this.imgLstToolBtn;
			this.tlbTool.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("tlbTool.ImeMode")));
			this.tlbTool.Location = ((System.Drawing.Point)(resources.GetObject("tlbTool.Location")));
			this.tlbTool.Name = "tlbTool";
			this.tlbTool.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("tlbTool.RightToLeft")));
			this.tlbTool.ShowToolTips = ((bool)(resources.GetObject("tlbTool.ShowToolTips")));
			this.tlbTool.Size = ((System.Drawing.Size)(resources.GetObject("tlbTool.Size")));
			this.tlbTool.TabIndex = ((int)(resources.GetObject("tlbTool.TabIndex")));
			this.tlbTool.TextAlign = ((System.Windows.Forms.ToolBarTextAlign)(resources.GetObject("tlbTool.TextAlign")));
			this.tlbTool.Visible = ((bool)(resources.GetObject("tlbTool.Visible")));
			this.tlbTool.Wrappable = ((bool)(resources.GetObject("tlbTool.Wrappable")));
			this.tlbTool.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tlbTool_ButtonClick);
			// 
			// tlbOpen
			// 
			this.tlbOpen.Enabled = ((bool)(resources.GetObject("tlbOpen.Enabled")));
			this.tlbOpen.ImageIndex = ((int)(resources.GetObject("tlbOpen.ImageIndex")));
			this.tlbOpen.Text = resources.GetString("tlbOpen.Text");
			this.tlbOpen.ToolTipText = resources.GetString("tlbOpen.ToolTipText");
			this.tlbOpen.Visible = ((bool)(resources.GetObject("tlbOpen.Visible")));
			// 
			// tlbMergeFiles
			// 
			this.tlbMergeFiles.Enabled = ((bool)(resources.GetObject("tlbMergeFiles.Enabled")));
			this.tlbMergeFiles.ImageIndex = ((int)(resources.GetObject("tlbMergeFiles.ImageIndex")));
			this.tlbMergeFiles.Tag = "";
			this.tlbMergeFiles.Text = resources.GetString("tlbMergeFiles.Text");
			this.tlbMergeFiles.ToolTipText = resources.GetString("tlbMergeFiles.ToolTipText");
			this.tlbMergeFiles.Visible = ((bool)(resources.GetObject("tlbMergeFiles.Visible")));
			// 
			// tlbZoomIn
			// 
			this.tlbZoomIn.Enabled = ((bool)(resources.GetObject("tlbZoomIn.Enabled")));
			this.tlbZoomIn.ImageIndex = ((int)(resources.GetObject("tlbZoomIn.ImageIndex")));
			this.tlbZoomIn.Text = resources.GetString("tlbZoomIn.Text");
			this.tlbZoomIn.ToolTipText = resources.GetString("tlbZoomIn.ToolTipText");
			this.tlbZoomIn.Visible = ((bool)(resources.GetObject("tlbZoomIn.Visible")));
			// 
			// tlbZoomOut
			// 
			this.tlbZoomOut.Enabled = ((bool)(resources.GetObject("tlbZoomOut.Enabled")));
			this.tlbZoomOut.ImageIndex = ((int)(resources.GetObject("tlbZoomOut.ImageIndex")));
			this.tlbZoomOut.Text = resources.GetString("tlbZoomOut.Text");
			this.tlbZoomOut.ToolTipText = resources.GetString("tlbZoomOut.ToolTipText");
			this.tlbZoomOut.Visible = ((bool)(resources.GetObject("tlbZoomOut.Visible")));
			// 
			// tlbSave
			// 
			this.tlbSave.Enabled = ((bool)(resources.GetObject("tlbSave.Enabled")));
			this.tlbSave.ImageIndex = ((int)(resources.GetObject("tlbSave.ImageIndex")));
			this.tlbSave.Text = resources.GetString("tlbSave.Text");
			this.tlbSave.ToolTipText = resources.GetString("tlbSave.ToolTipText");
			this.tlbSave.Visible = ((bool)(resources.GetObject("tlbSave.Visible")));
			// 
			// tlbWhite
			// 
			this.tlbWhite.Enabled = ((bool)(resources.GetObject("tlbWhite.Enabled")));
			this.tlbWhite.ImageIndex = ((int)(resources.GetObject("tlbWhite.ImageIndex")));
			this.tlbWhite.Text = resources.GetString("tlbWhite.Text");
			this.tlbWhite.ToolTipText = resources.GetString("tlbWhite.ToolTipText");
			this.tlbWhite.Visible = ((bool)(resources.GetObject("tlbWhite.Visible")));
			// 
			// tlbBlack
			// 
			this.tlbBlack.Enabled = ((bool)(resources.GetObject("tlbBlack.Enabled")));
			this.tlbBlack.ImageIndex = ((int)(resources.GetObject("tlbBlack.ImageIndex")));
			this.tlbBlack.Text = resources.GetString("tlbBlack.Text");
			this.tlbBlack.ToolTipText = resources.GetString("tlbBlack.ToolTipText");
			this.tlbBlack.Visible = ((bool)(resources.GetObject("tlbBlack.Visible")));
			// 
			// tlbColor
			// 
			this.tlbColor.Enabled = ((bool)(resources.GetObject("tlbColor.Enabled")));
			this.tlbColor.ImageIndex = ((int)(resources.GetObject("tlbColor.ImageIndex")));
			this.tlbColor.Text = resources.GetString("tlbColor.Text");
			this.tlbColor.ToolTipText = resources.GetString("tlbColor.ToolTipText");
			this.tlbColor.Visible = ((bool)(resources.GetObject("tlbColor.Visible")));
			// 
			// tlb3d
			// 
			this.tlb3d.DropDownMenu = this.orbitViewMenu;
			this.tlb3d.Enabled = ((bool)(resources.GetObject("tlb3d.Enabled")));
			this.tlb3d.ImageIndex = ((int)(resources.GetObject("tlb3d.ImageIndex")));
			this.tlb3d.Style = System.Windows.Forms.ToolBarButtonStyle.DropDownButton;
			this.tlb3d.Text = resources.GetString("tlb3d.Text");
			this.tlb3d.ToolTipText = resources.GetString("tlb3d.ToolTipText");
			this.tlb3d.Visible = ((bool)(resources.GetObject("tlb3d.Visible")));
			// 
			// orbitViewMenu
			// 
			this.orbitViewMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.InitialViewMenuItem,
																						  this.TopViewMenuItem,
																						  this.BottomViewMenuItem,
																						  this.LeftViewMenuItem,
																						  this.RightViewMenuItem,
																						  this.FrontViewMenuItem,
																						  this.BackViewMenuItem,
																						  this.SWViewMenuItem,
																						  this.SEViewMenuItem,
																						  this.NEViewMenuItem,
																						  this.NWViewMenuItem});
			this.orbitViewMenu.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("orbitViewMenu.RightToLeft")));
			// 
			// InitialViewMenuItem
			// 
			this.InitialViewMenuItem.Enabled = ((bool)(resources.GetObject("InitialViewMenuItem.Enabled")));
			this.InitialViewMenuItem.Index = 0;
			this.InitialViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("InitialViewMenuItem.Shortcut")));
			this.InitialViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("InitialViewMenuItem.ShowShortcut")));
			this.InitialViewMenuItem.Text = resources.GetString("InitialViewMenuItem.Text");
			this.InitialViewMenuItem.Visible = ((bool)(resources.GetObject("InitialViewMenuItem.Visible")));
			this.InitialViewMenuItem.Click += new System.EventHandler(this.InitialViewMenuItem_Click);
			// 
			// TopViewMenuItem
			// 
			this.TopViewMenuItem.Enabled = ((bool)(resources.GetObject("TopViewMenuItem.Enabled")));
			this.TopViewMenuItem.Index = 1;
			this.TopViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("TopViewMenuItem.Shortcut")));
			this.TopViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("TopViewMenuItem.ShowShortcut")));
			this.TopViewMenuItem.Text = resources.GetString("TopViewMenuItem.Text");
			this.TopViewMenuItem.Visible = ((bool)(resources.GetObject("TopViewMenuItem.Visible")));
			this.TopViewMenuItem.Click += new System.EventHandler(this.TopViewMenuItem_Click);
			// 
			// BottomViewMenuItem
			// 
			this.BottomViewMenuItem.Enabled = ((bool)(resources.GetObject("BottomViewMenuItem.Enabled")));
			this.BottomViewMenuItem.Index = 2;
			this.BottomViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("BottomViewMenuItem.Shortcut")));
			this.BottomViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("BottomViewMenuItem.ShowShortcut")));
			this.BottomViewMenuItem.Text = resources.GetString("BottomViewMenuItem.Text");
			this.BottomViewMenuItem.Visible = ((bool)(resources.GetObject("BottomViewMenuItem.Visible")));
			this.BottomViewMenuItem.Click += new System.EventHandler(this.BottomViewMenuItem_Click);
			// 
			// LeftViewMenuItem
			// 
			this.LeftViewMenuItem.Enabled = ((bool)(resources.GetObject("LeftViewMenuItem.Enabled")));
			this.LeftViewMenuItem.Index = 3;
			this.LeftViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("LeftViewMenuItem.Shortcut")));
			this.LeftViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("LeftViewMenuItem.ShowShortcut")));
			this.LeftViewMenuItem.Text = resources.GetString("LeftViewMenuItem.Text");
			this.LeftViewMenuItem.Visible = ((bool)(resources.GetObject("LeftViewMenuItem.Visible")));
			this.LeftViewMenuItem.Click += new System.EventHandler(this.LeftViewMenuItem_Click);
			// 
			// RightViewMenuItem
			// 
			this.RightViewMenuItem.Enabled = ((bool)(resources.GetObject("RightViewMenuItem.Enabled")));
			this.RightViewMenuItem.Index = 4;
			this.RightViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("RightViewMenuItem.Shortcut")));
			this.RightViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("RightViewMenuItem.ShowShortcut")));
			this.RightViewMenuItem.Text = resources.GetString("RightViewMenuItem.Text");
			this.RightViewMenuItem.Visible = ((bool)(resources.GetObject("RightViewMenuItem.Visible")));
			this.RightViewMenuItem.Click += new System.EventHandler(this.RightViewMenuItem_Click);
			// 
			// FrontViewMenuItem
			// 
			this.FrontViewMenuItem.Enabled = ((bool)(resources.GetObject("FrontViewMenuItem.Enabled")));
			this.FrontViewMenuItem.Index = 5;
			this.FrontViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("FrontViewMenuItem.Shortcut")));
			this.FrontViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("FrontViewMenuItem.ShowShortcut")));
			this.FrontViewMenuItem.Text = resources.GetString("FrontViewMenuItem.Text");
			this.FrontViewMenuItem.Visible = ((bool)(resources.GetObject("FrontViewMenuItem.Visible")));
			this.FrontViewMenuItem.Click += new System.EventHandler(this.FrontViewMenuItem_Click);
			// 
			// BackViewMenuItem
			// 
			this.BackViewMenuItem.Enabled = ((bool)(resources.GetObject("BackViewMenuItem.Enabled")));
			this.BackViewMenuItem.Index = 6;
			this.BackViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("BackViewMenuItem.Shortcut")));
			this.BackViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("BackViewMenuItem.ShowShortcut")));
			this.BackViewMenuItem.Text = resources.GetString("BackViewMenuItem.Text");
			this.BackViewMenuItem.Visible = ((bool)(resources.GetObject("BackViewMenuItem.Visible")));
			this.BackViewMenuItem.Click += new System.EventHandler(this.BackViewMenuItem_Click);
			// 
			// SWViewMenuItem
			// 
			this.SWViewMenuItem.Enabled = ((bool)(resources.GetObject("SWViewMenuItem.Enabled")));
			this.SWViewMenuItem.Index = 7;
			this.SWViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("SWViewMenuItem.Shortcut")));
			this.SWViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("SWViewMenuItem.ShowShortcut")));
			this.SWViewMenuItem.Text = resources.GetString("SWViewMenuItem.Text");
			this.SWViewMenuItem.Visible = ((bool)(resources.GetObject("SWViewMenuItem.Visible")));
			this.SWViewMenuItem.Click += new System.EventHandler(this.SWViewMenuItem_Click);
			// 
			// SEViewMenuItem
			// 
			this.SEViewMenuItem.Enabled = ((bool)(resources.GetObject("SEViewMenuItem.Enabled")));
			this.SEViewMenuItem.Index = 8;
			this.SEViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("SEViewMenuItem.Shortcut")));
			this.SEViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("SEViewMenuItem.ShowShortcut")));
			this.SEViewMenuItem.Text = resources.GetString("SEViewMenuItem.Text");
			this.SEViewMenuItem.Visible = ((bool)(resources.GetObject("SEViewMenuItem.Visible")));
			this.SEViewMenuItem.Click += new System.EventHandler(this.SEViewMenuItem_Click);
			// 
			// NEViewMenuItem
			// 
			this.NEViewMenuItem.Enabled = ((bool)(resources.GetObject("NEViewMenuItem.Enabled")));
			this.NEViewMenuItem.Index = 9;
			this.NEViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("NEViewMenuItem.Shortcut")));
			this.NEViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("NEViewMenuItem.ShowShortcut")));
			this.NEViewMenuItem.Text = resources.GetString("NEViewMenuItem.Text");
			this.NEViewMenuItem.Visible = ((bool)(resources.GetObject("NEViewMenuItem.Visible")));
			this.NEViewMenuItem.Click += new System.EventHandler(this.NEViewMenuItem_Click);
			// 
			// NWViewMenuItem
			// 
			this.NWViewMenuItem.Enabled = ((bool)(resources.GetObject("NWViewMenuItem.Enabled")));
			this.NWViewMenuItem.Index = 10;
			this.NWViewMenuItem.Shortcut = ((System.Windows.Forms.Shortcut)(resources.GetObject("NWViewMenuItem.Shortcut")));
			this.NWViewMenuItem.ShowShortcut = ((bool)(resources.GetObject("NWViewMenuItem.ShowShortcut")));
			this.NWViewMenuItem.Text = resources.GetString("NWViewMenuItem.Text");
			this.NWViewMenuItem.Visible = ((bool)(resources.GetObject("NWViewMenuItem.Visible")));
			this.NWViewMenuItem.Click += new System.EventHandler(this.NWViewMenuItem_Click);
			// 
			// tlbOSX
			// 
			this.tlbOSX.Enabled = ((bool)(resources.GetObject("tlbOSX.Enabled")));
			this.tlbOSX.ImageIndex = ((int)(resources.GetObject("tlbOSX.ImageIndex")));
			this.tlbOSX.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.tlbOSX.Text = resources.GetString("tlbOSX.Text");
			this.tlbOSX.ToolTipText = resources.GetString("tlbOSX.ToolTipText");
			this.tlbOSX.Visible = ((bool)(resources.GetObject("tlbOSX.Visible")));
			// 
			// tlbBlackMode
			// 
			this.tlbBlackMode.Enabled = ((bool)(resources.GetObject("tlbBlackMode.Enabled")));
			this.tlbBlackMode.ImageIndex = ((int)(resources.GetObject("tlbBlackMode.ImageIndex")));
			this.tlbBlackMode.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.tlbBlackMode.Text = resources.GetString("tlbBlackMode.Text");
			this.tlbBlackMode.ToolTipText = resources.GetString("tlbBlackMode.ToolTipText");
			this.tlbBlackMode.Visible = ((bool)(resources.GetObject("tlbBlackMode.Visible")));
			// 
			// tlbColorMode
			// 
			this.tlbColorMode.Enabled = ((bool)(resources.GetObject("tlbColorMode.Enabled")));
			this.tlbColorMode.ImageIndex = ((int)(resources.GetObject("tlbColorMode.ImageIndex")));
			this.tlbColorMode.Pushed = true;
			this.tlbColorMode.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.tlbColorMode.Text = resources.GetString("tlbColorMode.Text");
			this.tlbColorMode.ToolTipText = resources.GetString("tlbColorMode.ToolTipText");
			this.tlbColorMode.Visible = ((bool)(resources.GetObject("tlbColorMode.Visible")));
			// 
			// tlbDefSize
			// 
			this.tlbDefSize.Enabled = ((bool)(resources.GetObject("tlbDefSize.Enabled")));
			this.tlbDefSize.ImageIndex = ((int)(resources.GetObject("tlbDefSize.ImageIndex")));
			this.tlbDefSize.Text = resources.GetString("tlbDefSize.Text");
			this.tlbDefSize.ToolTipText = resources.GetString("tlbDefSize.ToolTipText");
			this.tlbDefSize.Visible = ((bool)(resources.GetObject("tlbDefSize.Visible")));
			// 
			// tlbTextShow
			// 
			this.tlbTextShow.Enabled = ((bool)(resources.GetObject("tlbTextShow.Enabled")));
			this.tlbTextShow.ImageIndex = ((int)(resources.GetObject("tlbTextShow.ImageIndex")));
			this.tlbTextShow.Pushed = true;
			this.tlbTextShow.Text = resources.GetString("tlbTextShow.Text");
			this.tlbTextShow.ToolTipText = resources.GetString("tlbTextShow.ToolTipText");
			this.tlbTextShow.Visible = ((bool)(resources.GetObject("tlbTextShow.Visible")));
			// 
			// tlbDimShow
			// 
			this.tlbDimShow.Enabled = ((bool)(resources.GetObject("tlbDimShow.Enabled")));
			this.tlbDimShow.ImageIndex = ((int)(resources.GetObject("tlbDimShow.ImageIndex")));
			this.tlbDimShow.Pushed = true;
			this.tlbDimShow.Text = resources.GetString("tlbDimShow.Text");
			this.tlbDimShow.ToolTipText = resources.GetString("tlbDimShow.ToolTipText");
			this.tlbDimShow.Visible = ((bool)(resources.GetObject("tlbDimShow.Visible")));
			// 
			// tlbSaveImageClip
			// 
			this.tlbSaveImageClip.Enabled = ((bool)(resources.GetObject("tlbSaveImageClip.Enabled")));
			this.tlbSaveImageClip.ImageIndex = ((int)(resources.GetObject("tlbSaveImageClip.ImageIndex")));
			this.tlbSaveImageClip.Text = resources.GetString("tlbSaveImageClip.Text");
			this.tlbSaveImageClip.ToolTipText = resources.GetString("tlbSaveImageClip.ToolTipText");
			this.tlbSaveImageClip.Visible = ((bool)(resources.GetObject("tlbSaveImageClip.Visible")));
			// 
			// tlbPrint
			// 
			this.tlbPrint.Enabled = ((bool)(resources.GetObject("tlbPrint.Enabled")));
			this.tlbPrint.ImageIndex = ((int)(resources.GetObject("tlbPrint.ImageIndex")));
			this.tlbPrint.Text = resources.GetString("tlbPrint.Text");
			this.tlbPrint.ToolTipText = resources.GetString("tlbPrint.ToolTipText");
			this.tlbPrint.Visible = ((bool)(resources.GetObject("tlbPrint.Visible")));
			// 
			// tlbRect
			// 
			this.tlbRect.Enabled = ((bool)(resources.GetObject("tlbRect.Enabled")));
			this.tlbRect.ImageIndex = ((int)(resources.GetObject("tlbRect.ImageIndex")));
			this.tlbRect.Text = resources.GetString("tlbRect.Text");
			this.tlbRect.ToolTipText = resources.GetString("tlbRect.ToolTipText");
			this.tlbRect.Visible = ((bool)(resources.GetObject("tlbRect.Visible")));
			// 
			// cadPictBox
			// 
			this.cadPictBox.AccessibleDescription = resources.GetString("cadPictBox.AccessibleDescription");
			this.cadPictBox.AccessibleName = resources.GetString("cadPictBox.AccessibleName");
			this.cadPictBox.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("cadPictBox.Anchor")));
			this.cadPictBox.BackColor = System.Drawing.Color.Black;
			this.cadPictBox.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("cadPictBox.BackgroundImage")));
			this.cadPictBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.cadPictBox.Cursor = System.Windows.Forms.Cursors.Default;
			this.cadPictBox.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("cadPictBox.Dock")));
			this.cadPictBox.DoubleBuffering = true;
			this.cadPictBox.Enabled = ((bool)(resources.GetObject("cadPictBox.Enabled")));
			this.cadPictBox.Font = ((System.Drawing.Font)(resources.GetObject("cadPictBox.Font")));
			this.cadPictBox.Image = ((System.Drawing.Image)(resources.GetObject("cadPictBox.Image")));
			this.cadPictBox.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("cadPictBox.ImeMode")));
			this.cadPictBox.Location = ((System.Drawing.Point)(resources.GetObject("cadPictBox.Location")));
			this.cadPictBox.Name = "cadPictBox";
			this.cadPictBox.Position = new System.Drawing.Point(0, 0);
			this.cadPictBox.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("cadPictBox.RightToLeft")));
			this.cadPictBox.Size = ((System.Drawing.Size)(resources.GetObject("cadPictBox.Size")));
			this.cadPictBox.SizeMode = ((System.Windows.Forms.PictureBoxSizeMode)(resources.GetObject("cadPictBox.SizeMode")));
			this.cadPictBox.TabIndex = ((int)(resources.GetObject("cadPictBox.TabIndex")));
			this.cadPictBox.TabStop = false;
			this.cadPictBox.Text = resources.GetString("cadPictBox.Text");
			this.cadPictBox.VirtualSize = new System.Drawing.Size(0, 0);
			this.cadPictBox.Visible = ((bool)(resources.GetObject("cadPictBox.Visible")));
			this.cadPictBox.MouseUp += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseUp);
			this.cadPictBox.Paint += new System.Windows.Forms.PaintEventHandler(this.cadPictBox_Paint);
			this.cadPictBox.MouseMove += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseMove);
			this.cadPictBox.MouseDown += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseDown);
			// 
			// printPrevDlg
			// 
			this.printPrevDlg.AccessibleDescription = resources.GetString("printPrevDlg.AccessibleDescription");
			this.printPrevDlg.AccessibleName = resources.GetString("printPrevDlg.AccessibleName");
			this.printPrevDlg.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("printPrevDlg.Anchor")));
			this.printPrevDlg.AutoScaleBaseSize = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.AutoScaleBaseSize")));
			this.printPrevDlg.AutoScroll = ((bool)(resources.GetObject("printPrevDlg.AutoScroll")));
			this.printPrevDlg.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.AutoScrollMargin")));
			this.printPrevDlg.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.AutoScrollMinSize")));
			this.printPrevDlg.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("printPrevDlg.BackgroundImage")));
			this.printPrevDlg.ClientSize = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.ClientSize")));
			this.printPrevDlg.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("printPrevDlg.Dock")));
			this.printPrevDlg.Enabled = ((bool)(resources.GetObject("printPrevDlg.Enabled")));
			this.printPrevDlg.Font = ((System.Drawing.Font)(resources.GetObject("printPrevDlg.Font")));
			this.printPrevDlg.Icon = ((System.Drawing.Icon)(resources.GetObject("printPrevDlg.Icon")));
			this.printPrevDlg.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("printPrevDlg.ImeMode")));
			this.printPrevDlg.Location = ((System.Drawing.Point)(resources.GetObject("printPrevDlg.Location1")));
			this.printPrevDlg.MaximumSize = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.MaximumSize")));
			this.printPrevDlg.MinimumSize = ((System.Drawing.Size)(resources.GetObject("printPrevDlg.MinimumSize")));
			this.printPrevDlg.Name = "printPrevDlg";
			this.printPrevDlg.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("printPrevDlg.RightToLeft")));
			this.printPrevDlg.StartPosition = ((System.Windows.Forms.FormStartPosition)(resources.GetObject("printPrevDlg.StartPosition")));
			this.printPrevDlg.Text = resources.GetString("printPrevDlg.Text");
			this.printPrevDlg.TransparencyKey = System.Drawing.Color.Empty;
			this.printPrevDlg.Visible = ((bool)(resources.GetObject("printPrevDlg.Visible")));
			// 
			// dlgOpenFile
			// 
			this.dlgOpenFile.DefaultExt = "*.dxf;*.dwg";
			this.dlgOpenFile.Filter = resources.GetString("dlgOpenFile.Filter");
			this.dlgOpenFile.ShowHelp = true;
			this.dlgOpenFile.Title = resources.GetString("dlgOpenFile.Title");
			// 
			// dlgSaveDXF
			// 
			this.dlgSaveDXF.DefaultExt = "*.dxf";
			this.dlgSaveDXF.Filter = resources.GetString("dlgSaveDXF.Filter");
			this.dlgSaveDXF.Title = resources.GetString("dlgSaveDXF.Title");
			// 
			// imgLstToolBtn
			// 
			this.imgLstToolBtn.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imgLstToolBtn.ImageSize = ((System.Drawing.Size)(resources.GetObject("imgLstToolBtn.ImageSize")));
			this.imgLstToolBtn.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgLstToolBtn.ImageStream")));
			this.imgLstToolBtn.TransparentColor = System.Drawing.Color.Magenta;
			// 
			// MainForm
			// 
			this.AccessibleDescription = resources.GetString("$this.AccessibleDescription");
			this.AccessibleName = resources.GetString("$this.AccessibleName");
			this.AutoScaleBaseSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScaleBaseSize")));
			this.AutoScroll = ((bool)(resources.GetObject("$this.AutoScroll")));
			this.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMargin")));
			this.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMinSize")));
			this.BackColor = System.Drawing.SystemColors.Control;
			this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
			this.ClientSize = ((System.Drawing.Size)(resources.GetObject("$this.ClientSize")));
			this.Controls.Add(this.cadPictBox);
			this.Controls.Add(this.tlbTool);
			this.Controls.Add(this.stBar);
			this.Enabled = ((bool)(resources.GetObject("$this.Enabled")));
			this.Font = ((System.Drawing.Font)(resources.GetObject("$this.Font")));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("$this.ImeMode")));
			this.Location = ((System.Drawing.Point)(resources.GetObject("$this.Location")));
			this.MaximumSize = ((System.Drawing.Size)(resources.GetObject("$this.MaximumSize")));
			this.Menu = this.mainMenu;
			this.MinimumSize = ((System.Drawing.Size)(resources.GetObject("$this.MinimumSize")));
			this.Name = "MainForm";
			this.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("$this.RightToLeft")));
			this.StartPosition = ((System.Windows.Forms.FormStartPosition)(resources.GetObject("$this.StartPosition")));
			this.Text = resources.GetString("$this.Text");
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Resize += new System.EventHandler(this.MainForm_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.Deactivate += new System.EventHandler(this.MainForm_Deactivate);
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion
				
		private static void LoadParams(string[] args)
		{
			if(args.Length != 0)	
				openFileName = args[0];							
		}

		static MainForm()
		{
			fileSettingsName = Application.StartupPath + @"\Settings.txt";
			openFileName = string.Empty;
		}

		[STAThread]
		static void Main(string[] args) 
		{
			Application.EnableVisualStyles();
			Application.DoEvents();			
			LoadParams(args);
			Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(DoExceptionWorking);
			Application.Run(new MainForm());			
		}

		private static void DoExceptionWorking(object sender, System.Threading.ThreadExceptionEventArgs args)
		{				
			CADConst.SendExceptionReport(args.Exception);
		}

		#region Help
		/// <summary>
		/// Adjusts the image size according to the specified scale.
		/// </summary>
		/// <param name="i">A scale value.</param>
		/// <remarks>If the scale value is more than one the image size increases; otherwise, decreases.</remarks>
		#endregion Help
		public void Zoom(float i)
		{
			if(cadImage == null) return;
			sc = sc * i;
			if(sc < 0.005f) 
				sc = 0.005f;	
			cadPictBox.Invalidate();
			SetPictureBoxPosition(Position);
			cadPictBox_MouseMove(cadPictBox, new MouseEventArgs(MouseButtons.Left, 0, (int)old_Pos.X, (int)old_Pos.Y, 0));		
		}

		private void Shift()
		{
			LeftImagePosition = PreviousPosition.X - (PreviousPosition.X - LeftImagePosition) * ImageScale / ImagePreviousScale;
			TopImagePosition = PreviousPosition.Y - (PreviousPosition.Y - TopImagePosition) * ImageScale / ImagePreviousScale;
			ImagePreviousScale = ImageScale;		
		}

		private void CADPictBoxMouseWheel(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(e.Delta < 0)	
				Zoom(0.7f);
			else 
				Zoom(1.3f);
			this.Shift();			
		}

		private void SetPictureBoxPosition(PointF value)
		{
			int w1, h1;
			if(value.X > 0)
				w1 = 0;
			else
				w1 = (int)Math.Abs(value.X);
			if(w1 > this.cadPictBox.VirtualSize.Width)
				w1 = this.cadPictBox.VirtualSize.Width;
			if(value.Y > 0)
				h1 = 0;
			else
				h1 = (int)Math.Abs(value.Y);
			if(h1 > this.cadPictBox.VirtualSize.Height)
				h1 = this.cadPictBox.VirtualSize.Height;
			this.cadPictBox.SetPositionNoInvalidate(new Point(w1, h1));
		}

		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref="MainForm">MainForm</see> class.
		/// </summary>
		#endregion Help
		public MainForm()
		{		
			InitParams();									
			InitializeComponent();									
			InitExtParam();
		}

		private void InitParams()
		{			
			ImagePreviousScale = 1.0f;			
			curLngIndex = 0;
			ImageScale = 1;			
			dimVisible = true;			
			drawMode = true;
			showLineWeight = true;
			textVisible = true;
			PreviousPosition = new PointF();			
			Orb3D = new Orbit3D();
			this.Orb3D.RotateEvent += new CADImport.FaceModule.CADImportEventHandler(this.ResizeLayout);
			lForm = new LayerForm();			
			aboutFrm = new AboutForm();
			shxFrm = new SHXForm();
			mergeForm = new MergeForm();
			rasterSizeForm = new SetRasterSizeForm();
			openFromWebForm = new OpenFileFromWebForm();
			#region protect
#if (protect && floatprotect)
			floatLicFrm = new FloatLicForm();
			floatLicFrm.Change += new System.EventHandler(InvalidateImage);
#endif
#if protect
			regFrm = new RegForm();
#endif
			#endregion protect			
			prtForm = new CADImport.Printing.PrintingForm();
			this.prtForm.LayerForm = lForm;
			listItemSelectDel =  new ListItemSelectEventHandler(ListSelect);				
			dlgPrintPrev = new PrintPreviewDialog();
		}

		private void InitExtParam()
		{
			Initial3DOrbit();
			#region protect
#if ((! floatprotect) && protect)
				this.miFloatLic.Enabled = false;
#endif
			#endregion						
			this.clipRectangle = new ClipRect(this.cadPictBox);		
			this.clipRectangle.MultySelect = false;			
			this.cadPictBox.MouseWheel += new MouseEventHandler(CADPictBoxMouseWheel);
			this.cadPictBox.ScrollEvent += new CADImport.FaceModule.ScrollEventHandlerExt(CADPictBoxScroll);		
			this.mergeForm.ResizeEvent += new CADImport.FaceModule.CADImportEventHandler(this.DoResize);
		}

		private void ResizeLayout()
		{
			if(this.cadImage == null) return;
			this.cadImage.GetExtents();
			this.DoResize(); 
		}				

		private void CADPictBoxScroll(object sender, CADImport.FaceModule.ScrollEventArgsExt e)
		{			
			LeftImagePosition = -this.cadPictBox.Position.X;
			TopImagePosition = -this.cadPictBox.Position.Y;
			this.cadPictBox.Invalidate();
		}		

		private void ChangeDrawingColors(bool val)
		{
			if(val)
				this.DoNormalColor();				
			else
				this.DoBlackColor();				
		}

		private void ChangeBackColor(bool val)
		{
			if(val)
				this.Black_Click();
			else
				this.White_Click();
		}						

		private void InvalidateImage(object sender, System.EventArgs e)
		{
			this.cadPictBox.Invalidate();
		}

		private void Initial3DOrbit()
		{
			Orb3D.Parent = this.cadPictBox;
		}
		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private void Open_Click(object sender, System.EventArgs e)
		{
			this.ShowLoadFileDialog();
			this.ChangeControlState();
		}

		private void DeactCADImage()
		{			
			lForm.LayerList.Clear();
			lForm.LayerList.SelectItemEvent -= listItemSelectDel;			
		}

		#region Help 
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
		/// <summary>
		/// Invokes open file dialog and loads specified file
		/// </summary>
#endif
		#endregion
		public void ShowLoadFileDialog()
		{
			DeactCADImage();
			this.cadPictBox.Enabled = false;
			if(dlgOpenFile.ShowDialog() != DialogResult.OK) 
			{
				cadPictBox.Visible = true;	
				this.cadPictBox.Enabled = true;
				return;
			}
			this.cadPictBox.Enabled = true;
			LoadFile(dlgOpenFile.FileName);			
		}

		#region Help
		/// <summary>
		/// Loads a CAD file.
		/// </summary>
		#endregion Help
		public void LoadFile(string fileName)
		{									
			if(fileName != null)
			{
				if(cadImage != null)
				{	
					cadImage.Dispose();
					cadImage = null;
					this.lForm.LayerList.Clear();
					lForm.LayerList.SelectItemEvent -= listItemSelectDel;			
				}
				this.Cursor = Cursors.WaitCursor;
				stBar.Panels[0].Text = ApplicationConstants.loadfilestr;
				ImageScale = 1.0f;
				ImagePreviousScale = 1.0f;
				Position = new PointF();				
				this.cadImage = CADImage.CreateImageByExtension(fileName);																
				if(this.cadImage is CADRasterImage)
					(this.cadImage as CADRasterImage).Control = this.cadPictBox;
			} 
			if(this.cadImage != null)
			{				
				if(CADConst.IsWebPath(fileName))
					this.cadImage.LoadFromWeb(fileName);
				else
					cadImage.LoadFromFile(fileName);								
			}
			stBar.Panels[0].Text = Path.GetFileName(fileName) + ApplicationConstants.sepstr;			
			CADImage.LastLoadedFilePath = Path.GetDirectoryName(fileName); 
			SetCADImageOptions();
		}

		#region Help
		/// <summary>
		/// Alternately changes a boolean value that indicates if a thickness of lines is determined by the file data.
		/// </summary>
		#endregion Help
		public void ChangeShowLineWeight()
		{				
			this.showLineWeight = ! this.showLineWeight;
			cadImage.IsShowLineWeight = this.showLineWeight;
			this.cadPictBox.Invalidate();
		}

		#region Help
#if RUSHELP
		/// <summary>
		///     <see cref="MainForm.Image"/>CADImage    
		/// </summary>
#else
		/// <summary>
		/// Sets current properties for <see cref="MainForm.Image"/>CADImage for loading file
		/// </summary>
#endif
		#endregion
		public void SetCADImageOptions()
		{			
			cadImage.UseWinEllipse = false;		
			this.cadImage.UseDoubleBuffering = false;
			cadImage.IsShowLineWeight = false;
			cadImage.IsShowLineWeight = this.showLineWeight;		
			this.showLineWeight = false;									
			Orb3D.CADImage = cadImage;
			Orb3D.Visible = false;
			Orb3D.Disable3dOrbit();
			if(cadPictBox.BackColor == Color.White)
				White_Click();
			else 
				Black_Click();						
			this.Cursor = Cursors.Default;			
			if(this.DrawMode == false) 
				this.DoBlackColor();			 			
			SetLayList();	
			ObjEntity.cadImage = cadImage;			
			this.prtForm.Image = this.cadImage;		
			this.mergeForm.Image = this.cadImage;
			DoResize();			
			SetPictureBoxPosition(Position);		
		}

		private void NotSupportedInCurrentVersion()
		{
			this.Cursor = Cursors.Default;
			cadPictBox.Visible = true;
			stBar.Panels[0].Text = ApplicationConstants.notsupportedstr;
			MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr);
			return;
		}					

		#region Help
		/// <summary>
		/// Loads an empty image into the <see cref="MainForm">MainForm</see>.
		/// </summary>
		#endregion Help
		public void LoadEmptyImage()
		{			
			lForm.LayerList.Clear();
			lForm.LayerList.SelectItemEvent -= this.listItemSelectDel;			
			if(cadImage != null)
			{	
				cadImage.Dispose();
				cadImage = null;
			}			
			cadImage = new CADImage();
			cadImage.InitialNewImage();			
			tlbTool.Buttons[9].Pushed = cadImage.IsDraw3DAxes;
			tlbTool.Buttons[10].Pushed = false;
			tlbTool.Buttons[8].Pushed = false;
			tlbTool.Buttons[11].Pushed = true;			
			EnableButton(true);		
			this.SetCADImageOptions();
		}		

		#region Help
		/// <summary>
		/// Enables or disables buttons of the toolbar and menu items of the main menu.
		/// </summary>
		/// <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
		/// <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
		#endregion Help
		public void EnableButton(bool aVal)
		{
			for(int i = 2; i < 18; i++)
				tlbTool.Buttons[i].Enabled = aVal;			
			mSaveFile.Enabled = aVal;			
			closeMenuItem.Enabled = aVal;
			printMenuItem.Enabled = aVal;
			printprevItem.Enabled = aVal;
			print2Item.Enabled = aVal;
			#if Export
				this.mnSaveAsDXF.Enabled = aVal;
			#endif
		}

		private void ZoomIn_Click(object sender, System.EventArgs e)
		{
			DoZoomIn();
		}

		#region Help
		/// <summary>
		/// Increases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomIn()
		{
			if(cadImage == null) return;
			old_Pos = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			sc = sc * 2.0f;
			cadPictBox.Invalidate();	
			SetPictureBoxPosition(Position);
		}

		private void ZoomOut_Click(object sender, System.EventArgs e)
		{
			DoZoomOut();
		}

		#region Help
		/// <summary>
		/// Decreases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomOut()
		{
			if(cadImage == null) return;
			old_Pos = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			sc = sc / 2.0f;
			cadPictBox.Invalidate();	
			SetPictureBoxPosition(Position);
		}

		private void cadPictBox_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{				
			if(cadImage == null) return;			
			DrawCADImage(e.Graphics);
			stBar.Panels[1].Text = RealScale;										
		}

		#region Help
		/// <summary>
		/// Draws a CAD image from the loaded CAD file.
		/// </summary>
		/// <param name="gr">A <see cref="System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
		#endregion Help
		public void DrawCADImage(Graphics gr)
		{						
			try
			{								
				Shift();
				RectangleF tmp = ImageRectangleF;											
				SetSizePictureBox(new Size((int)tmp.Width, (int)tmp.Height));																								
				cadImage.Draw(gr, tmp);								
			}
			catch
			{
				return;
			}			
		}

		#region Help
#if RUSHELP
		/// <summary>
		///      <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz"> </param>
#else
		/// <summary>
		/// Sets scroll region for <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz">Size to set</param>
#endif
		#endregion
		public void SetSizePictureBox(Size sz)
		{		
			this.cadPictBox.SetVirtualSizeNoInvalidate(sz);			
		}


		private void cadPictBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			this.detRMouseDown = false;
			cadPictBox.Cursor = Cursors.Default;
			if(this.clipRectangle.Enabled)
				if((this.clipRectangle.Type == RectangleType.Zooming))
				{
					if((this.clipRectangle.ClientRectangle.Width <= 5)||
						(this.clipRectangle.ClientRectangle.Height <= 5)) 
					{ 
						cadPictBox.Invalidate(); 
						return;
					}
					if(this.sc > 1000) 
					{ 
						cadPictBox.Invalidate(); 
						return; 
					}
					float tmp1 = this.VisibleAreaSize.Width / this.clipRectangle.ClientRectangle.Width;
					float tmp2 = this.VisibleAreaSize.Height / this.clipRectangle.ClientRectangle.Height;
					if(tmp1 > tmp2) tmp1 = tmp2;
					pos.X += this.clipRectangle.ClientRectangle.Width / 2; 
					pos.Y += this.clipRectangle.ClientRectangle.Height / 2; 
					this.Zoom(tmp1);
				}
			cadPictBox.Invalidate();
		}

		private void cadPictBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if((e.Button == MouseButtons.Right)||(e.Button == MouseButtons.Middle))
			{
				MainForm.ActiveForm.Cursor = Cursors.Hand;
				cX = e.X;
				cY = e.Y;
				this.detRMouseDown = true;
			}
			else
			{
				if((! this.clipRectangle.Enabled)&&(this.cadImage != null)&&(! this.Orb3D.Visible))				
					this.clipRectangle.EnableRect(RectangleType.Zooming, new Rectangle(e.X, e.Y, 0, 0));				
			}
		}

		private void cadPictBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			cadPictBox.Focus();
			if(cadImage == null) return;
			if((this.detRMouseDown)&&(!Orb3D.Visible))
			{
				pos.X -= (cX - e.X);
				pos.Y -= (cY - e.Y);
				cX = e.X;
				cY = e.Y;
				cadPictBox.Invalidate();
			}
			old_Pos = new PointF(e.X, e.Y);
			DPoint vPt = GetRealPoint(e.X, e.Y);
			stBar.Panels[2].Text = vPt.X.ToString() + ApplicationConstants.sepstr2 + vPt.Y.ToString() + 
				ApplicationConstants.sepstr2 + vPt.Z.ToString();		
		}

		#region Help
		/// <summary>
		/// Gets a three-dimensional point of the current CAD image from the specified screen point.
		/// </summary>
		/// <param name="x">An X coordinate of the screen point.</param>
		/// <param name="y">A Y coordinate of the screen point.</param>
		/// <returns>A three-dimensional point.</returns>
		#endregion Help
		public DPoint GetRealPoint(int x, int y)
		{			
			RectangleF tmpRect = ImageRectangleF;	
			return CADConst.GetRealPoint(x, y, this.cadImage, tmpRect);
		}	

		#region Help
		/// <summary>
		/// Invokes a <see cref="System.Windows.Forms.SaveFileDialog">SaveFileDialog</see> box where 
		/// a user can select a directory and format for saving the current CAD image.
		/// </summary>
		/// <remarks><p>This method can be invoked from the toolbar and from the main menu.</p>
		/// <p>Images can be saved in the following formats: JPG, BMP, TIFF, GIF, EMF.</p></remarks>
		#endregion Help
		public void SaveAsImage()
		{
			if(cadImage == null) 
				return;
			if(this.cadImage.FileInf != null)
				dlgSaveImg.FileName = this.cadImage.FileInf.Name.Replace(this.cadImage.FileInf.Extension, string.Empty);
			if(dlgSaveImg.ShowDialog() != DialogResult.OK) 
				return;
			DRect tmpRect = new DRect(0.0, 0.0, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);
			ImageFormat imgFrm = ImageFormat.Bmp;
			if(dlgSaveImg.FileName != null) 
			{
				string tmp = dlgSaveImg.FileName;	
				if(tmp.ToUpper().IndexOf(ApplicationConstants.jpgextstr) != -1)
					imgFrm = ImageFormat.Jpeg;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.bmpextstr) != -1)
					imgFrm = ImageFormat.Bmp;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.tiffextstr) != -1)
					imgFrm = ImageFormat.Tiff;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.gifextstr) != -1)
					imgFrm = ImageFormat.Gif;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.emfextstr) != -1)
					imgFrm = ImageFormat.Emf;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.wmfextstr) != -1)
					imgFrm = ImageFormat.Wmf;
				if(tmp.ToUpper().IndexOf(ApplicationConstants.pngextstr) != -1)
					imgFrm = ImageFormat.Png;				
				if(tmp.ToUpper().IndexOf(ApplicationConstants.dxfextstr) != -1)
				{
					SaveAsDXF(dlgSaveImg.FileName);
					return;
				}
			}			
			if(! this.clipRectangle.Enabled)
			{
				rasterSizeForm.Image = this.cadImage;
				rasterSizeForm.CurrentSize = tmpRect;
				rasterSizeForm.SaveFileName = dlgSaveImg.FileName;
				if(rasterSizeForm.ShowDialog() == DialogResult.OK)
					tmpRect = rasterSizeForm.SizeImage;
			}						
			int tmpNumberOfPartsInCircle = this.cadImage.NumberOfPartsInCircle;
			this.cadImage.NumberOfPartsInCircle = CADConst.SetNumberOfPartsInCurve(tmpRect);
			if(this.clipRectangle.Enabled)
			{				
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))				
					cadImage.ExportToMetafile(dlgSaveImg.FileName, tmpRect, this.clipRectangle.ClientRectangle);				
				else
					cadImage.SaveToFile(dlgSaveImg.FileName, imgFrm, tmpRect, this.clipRectangle.ClientRectangle, rasterSizeForm.ImagePixelFormat);				
			}
			else
			{				
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))
					cadImage.ExportToMetafile(dlgSaveImg.FileName, tmpRect);
				else
					cadImage.SaveToFile(dlgSaveImg.FileName, imgFrm, tmpRect, rasterSizeForm.ImagePixelFormat);				
			}						
			this.cadImage.NumberOfPartsInCircle = tmpNumberOfPartsInCircle;
		}	

		private void menuItemZoom_Click(object sender, System.EventArgs e)
		{
			if(cadImage == null) return;
			float i = sc;
			switch((sender as MenuItem).Index)
			{
				case 0:
					i = 0.1f;
					break;
				case 1:
					i = 0.25f;
					break;
				case 2:
					i = 0.5f;
					break;
				case 3:
					i = 1.0f;
					break;
				case 4:
					i = 2.0f;
					break;
				case 5:
					i = 4.0f;
					break;
				case 6:
					i = 8.0f;
					break;
			}
			ResetScaling();
			old_Pos = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			sc = i;
			cadPictBox.Invalidate();		
			SetPictureBoxPosition(Position);
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the white color.
		/// </summary>
		#endregion Help
		public void White_Click()
		{			
			tlbTool.Buttons[5].Pushed = true;
			tlbTool.Buttons[6].Pushed = false;
			cadPictBox.BackColor = Color.White;
			if(cadImage != null)
			{
				cadImage.DefaultColor = Color.Black;
				cadImage.BackgroundColor = Color.White;
			}
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.Black;
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the black color.
		/// </summary>
		#endregion Help
		public void Black_Click()
		{			
			tlbTool.Buttons[5].Pushed = false;
			tlbTool.Buttons[6].Pushed = true;
			cadPictBox.BackColor = Color.Black;
			if(cadImage != null)
			{
				cadImage.DefaultColor = Color.White;
				cadImage.BackgroundColor = Color.Black;
			}
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.White;
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the user defined color.
		/// </summary>
		#endregion Help
		public void Color_Click()
		{
			if(dlgSetColor.ShowDialog() == DialogResult.OK)			
				cadPictBox.BackColor = dlgSetColor.Color;			
		}

		private void tlbTool_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if((cadImage == null)&&(tlbTool.Buttons.IndexOf(e.Button) == 8)) return;
			switch(tlbTool.Buttons.IndexOf(e.Button))
			{
				case 0:
					this.ShowLoadFileDialog();
					break; 
				case 1:
					if(this.cadImage == null)
						this.LoadEmptyImage();
					this.mergeForm.ShowDialog();					
					break;
				case 2:
					DoZoomIn();
					break; 
				case 3:
					DoZoomOut();
					break;
				case 4:
					SaveAsImage();
					break;
				case 5:
					White_Click();
					break;
				case 6:
					Black_Click();
					break;
				case 7:
					Color_Click();
					break;
				case 8:
					Go3dOrbit(! this.Orb3D.Visible);
					break;
				case 9:
					Change3DAxesVisiblity();
					break;
				case 10:
					DoBlackColor();
					break;
				case 11:
					DoNormalColor();
					break;
				case 12:
					ResetScaling();
					break;
				case 13:
					ChangeTextsVisiblity();
					break;
				case 14:
					ChangeDimensionsVisiblity();
					break;
				case 15:
					PutToClipboard();
					break;
				case 16:
					cadImage.Print(true, true);
					break;
				case 17:
					DoEnableRectangle(! this.clipRectangle.Enabled);
					break;
			}
			this.ChangeControlState();
		}

		private void ChangeControlState()
		{
			bool detNullImg = cadImage != null;
			EnableButton(detNullImg);
			bool tmp = false;
			if(cadPictBox.BackColor == Color.White)
				tmp = true;			
			if(detNullImg)
			{
				this.tlbWhite.Pushed = tmp;
				this.tlbBlack.Pushed = ! tmp;				
			}
			else 
			{
				this.tlbWhite.Pushed = false;
				this.tlbBlack.Pushed = false;				
			}
			tmp = this.textVisible;			
			if(detNullImg)
				this.tlbTextShow.Pushed = tmp;
			else
				this.tlbTextShow.Pushed = false;
			if(! tmp)
				this.tlbTextShow.ImageIndex = 15;
			else
				this.tlbTextShow.ImageIndex = 14;			
			tmp = this.dimVisible;
			if(detNullImg)
				this.tlbDimShow.Pushed = tmp;
			else 
				this.tlbDimShow.Pushed = false;
			if(! tmp)
				this.tlbDimShow.ImageIndex = 17;
			else 
				this.tlbDimShow.ImageIndex = 16;			
			tmp = this.Orb3D.Visible;			
			this.tlb3d.Pushed = tmp;						
			tmp = this.DrawMode;
			if(detNullImg)
			{				
				this.tlbBlackMode.Pushed = ! tmp;
				this.tlbColorMode.Pushed = tmp;
			}
			else
			{
				this.tlbBlackMode.Pushed = false;
				this.tlbColorMode.Pushed = false;
			}						
			if(curLngIndex < this.mainMenu.MenuItems[1].MenuItems[0].MenuItems.Count)
				this.mainMenu.MenuItems[1].MenuItems[0].MenuItems[curLngIndex].Checked = false;
			if(this.mainMenu.MenuItems[1].MenuItems[0].MenuItems.Count > this.curLngIndex)
				this.mainMenu.MenuItems[1].MenuItems[0].MenuItems[this.curLngIndex].Checked = true;						
		}

		#region Help
		/// <summary>
		/// Shows or hides texts in the current CAD image.
		/// </summary>
		#endregion Help
		public void ChangeTextsVisiblity()
		{
			this.textVisible = ! this.textVisible;
			this.cadImage.TextVisible = this.textVisible;
			cadPictBox.Invalidate();			
		}
		
		#region Help
		/// <summary>
		/// Shows or hides dimensions in the current CAD image.
		/// </summary>
		#endregion Help
		public void ChangeDimensionsVisiblity()
		{
			this.dimVisible = ! this.dimVisible;
			this.cadImage.DimensionsVisible = this.dimVisible;
			cadPictBox.Invalidate();
		}

		private void PutToClipboard()
		{
			if(this.clipRectangle.Enabled)
			{
				DRect tmpRect = ImageDRect;
				cadImage.SaveImageToClipboard(tmpRect, this.clipRectangle.ClientRectangle);
			}
			else
			{
				DRect tmp = new DRect(0.0, 0.0, VisibleAreaSize.Width*ImageScale, VisibleAreaSize.Height*ImageScale);
				cadImage.SaveImageToClipboard(tmp);
			}
		}

		private void DoEnableRectangle(bool val)
		{
			if(val)
			{
				Go3dOrbit(false);
				this.clipRectangle.Color = cadImage.DefaultColor;
				this.clipRectangle.EnableRect(RectangleType.Selecting);
			}
			else this.clipRectangle.DisableRect();
			tlbTool.Buttons[17].Pushed = val;
		}

		#region Help
		/// <summary>
		/// Shows or hides 3D axes.
		/// </summary>
		#endregion Help
		public void Change3DAxesVisiblity()
		{
			cadImage.IsDraw3DAxes = !cadImage.IsDraw3DAxes;
			tlbTool.Buttons[9].Pushed = cadImage.IsDraw3DAxes;
			cadPictBox.Invalidate();
		}
		

		#region Help
		/// <summary>
		/// Sets the scale and position of the current CAD image to its original values.
		/// </summary>
		#endregion Help
		public void ResetScaling()
		{
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - VisibleAreaSize.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - VisibleAreaSize.Height) / 2.0f;
			cadPictBox.Invalidate();
		}

		private void AddRecordToLayerList(string name, Color color, bool visible, bool freeze)
		{
			ListItem lstItem = new ListItem();			
			lstItem.Text = name;								
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(visible)				
				lstItem.ImageIndex = 7;							
			else
				lstItem.ImageIndex = 8;							
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(freeze)
				lstItem.ImageIndex = 3;				
			else
				lstItem.ImageIndex = 4;				
			lForm.LayerList.Items.Add(lstItem);	
			lstItem = new ListItem();
			lstItem.DrawColorRect = true;
			lstItem.BackColor = color;			
			lForm.LayerList.Items.Add(lstItem);			
		}
  
		private void SetLayList()
		{
			if((lForm.LayerList.Items.Count != 0)||(cadImage == null))
				return;
			ObjEntity.LayersList = new string[cadImage.LayersCount];			
			Color colorLay;					
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr1, ApplicationConstants.namestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr2, ApplicationConstants.visiblestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr4, ApplicationConstants.freezestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr3, ApplicationConstants.colorstr, 0));			
			string tmp;
			for(int I = 0; I < cadImage.LayersCount; I++)
			{
				tmp = cadImage.GetLayerName(I);
				colorLay = cadImage.GetLayerColor(I);
				ObjEntity.LayersList[I] = tmp;
				AddRecordToLayerList(tmp, colorLay, cadImage.GetLayerVisible(I), cadImage.GetLayerFreeze(I));				
			}		
			lForm.LayerList.SetAutoSize();
			lForm.LayerList.SelectItemEvent += listItemSelectDel;			
		}

		#region Help
		/// <summary>
		/// Alternately displays or hides a 3D Orbit tool.
		/// </summary>
		#endregion Help
		public void Go3dOrbit(bool val)
		{
			Orb3D.Visible = val;
			tlbTool.Buttons[7].Pushed = val;			
			this.detRMouseDown = false;
			if(val) 
			{
				DoEnableRectangle(false);
				Orb3D.Enable3dOrbit();
			}
			else
			{
				Orb3D.Disable3dOrbit();
			}
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Resizes the CAD image to fit drawing's bounds.
		/// </summary>
		/// <remarks>This method is invoked when changing layout or working with a 3D Orbit tool.</remarks>
		#endregion Help
		public void DoResize()
		{				
			if(cadImage == null) 
				return;
			float wh = (float)(cadImage.AbsWidth / cadImage.AbsHeight); 
			float new_wh = (float)cadPictBox.ClientRectangle.Width / (float)cadPictBox.ClientRectangle.Height;
			if(cadImage is CADRasterImage)
				visibleArea = new SizeF((float)cadImage.AbsWidth, (float)cadImage.AbsHeight);
			else 
				visibleArea = cadPictBox.Size;
			if(new_wh > wh)
				visibleArea.Width = visibleArea.Height * wh;
			else 
			{
				if(new_wh < wh)
					visibleArea.Height = visibleArea.Width / wh;
				else
					visibleArea = cadPictBox.Size;
			}
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - visibleArea.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - visibleArea.Height) / 2.0f;				
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Closes a currently open CAD file.
		/// </summary>
		#endregion Help
		public void CloseFile()
		{
			cadImage.Dispose();
			cadImage = null;
			EnableButton(false);			
			this.cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Displays a CAD image in all used colors.
		/// </summary>
		#endregion Help
		public void DoNormalColor()
		{
			if(cadImage == null) 
				return;
			cadImage.DrawMode = CADDrawMode.Normal;
			this.drawMode = true;
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Displays a CAD image in black and white colors.
		/// </summary>
		#endregion Help
		public void DoBlackColor()
		{
			if(cadImage == null) 
				return;
			cadImage.DrawMode = CADDrawMode.Black;
			this.drawMode = false;
			cadPictBox.Invalidate();
		}
		#region Help
		/// <summary>
		/// Makes the layers of the loaded CAD image visible or invisible.
		/// </summary>
		/// <param name="sender">A <see cref="System.Windows.Forms.CheckedListBox">CheckedListBox</see> that initializes changing a visibility of the layers.</param>
		/// <param name="e">An <see cref="System.Windows.Forms.ItemCheckEventArgs">ItemCheckEventArgs</see> object that provides data for the event of changing a layer visibility.</param>
		#endregion Help
		private void chLay_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			if(e.NewValue == CheckState.Checked)
				cadImage.SetLayerVisible(e.Index, true, false);
			else if(e.NewValue == CheckState.Unchecked) 
				cadImage.SetLayerVisible(e.Index, false, false);
			cadPictBox.Invalidate();
		}

		private void ImgSave_Click(object sender, System.EventArgs e)
		{
			SaveAsImage();
		}

		private void MainForm_Deactivate(object sender, System.EventArgs e)
		{
			this.detRMouseDown = false;
		}

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void aboutMenuItem_Click(object sender, System.EventArgs e)
		{
			aboutFrm.ShowDialog();
		}

		private void MainForm_Resize(object sender, System.EventArgs e)
		{
			DoResize();
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{					
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.DisableFloating();
			}
			else
			{
				Protection.CloseApplication();
			}
#else
			Protection.CloseApplication();
#endif
#endif
			#endregion
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{			
			this.VisibleAreaSize = cadPictBox.ClientSize;
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.FloatingLicRegister();
			}
			else	
			{
				Protection.Register();
			}
#else
			Protection.Register();
#endif
#endif
			#endregion
			if(openFileName != string.Empty)							
				this.LoadFile(openFileName);												
			this.ChangeControlState();
			#region demo
			#if Demo
				DemoForm d1 = new DemoForm();
				d1.ShowDialog();
			#endif
			#endregion demo
		}

		private void BackViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Back);
			cadPictBox.Invalidate();
		}

		private void TopViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Top);
			cadPictBox.Invalidate();
		}

		private void BottomViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Bottom);
			cadPictBox.Invalidate();
		}

		private void LeftViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Left);
			cadPictBox.Invalidate();
		}

		private void RightViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Right);
			cadPictBox.Invalidate();
		}

		private void FrontViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.Front);
			cadPictBox.Invalidate();
		}

		private void SWViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.SWIsometric);
			cadPictBox.Invalidate();
		}

		private void SEViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.SEIsometric);
			cadPictBox.Invalidate();
		}

		private void NEViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.NEIsometric);
			cadPictBox.Invalidate();
		}

		private void NWViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotToView(CADViewDirection.NWIsometric);
			cadPictBox.Invalidate();
		}

		private void InitialViewMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.RotDefault();
			cadPictBox.Invalidate();
		}

		private void toolMenuItem_Click(object sender, System.EventArgs e)
		{
			#region protect
			#if protect
			regFrm.ShowDialog();
			#endif
			#endregion protect
		}		

		private void fitMenuItem_Click(object sender, System.EventArgs e)
		{
			ResetScaling();
		}

		private void copyMenuItem_Click(object sender, System.EventArgs e)
		{
			DRect tmp = new DRect(0.0, 0.0, this.visibleArea.Width * sc, this.visibleArea.Height * sc);
			cadImage.SaveImageToClipboard(tmp);
		}

		private void closeMenuItem_Click(object sender, System.EventArgs e)
		{
			CloseFile();
		}

		private void printMenuItem_Click(object sender, System.EventArgs e)
		{
			cadImage.Print(true, true);
		}
		
		private void colorMenuItem_Click(object sender, System.EventArgs e)
		{
			DoNormalColor();
		}

		private void blackMenuItem_Click(object sender, System.EventArgs e)
		{
			DoBlackColor();
		}

		private void whiteBackMenuItem_Click(object sender, System.EventArgs e)
		{
			White_Click();
		}

		private void blackBackMenuItem_Click(object sender, System.EventArgs e)
		{
			Black_Click();
		}

		private void changeBackMenuItem_Click(object sender, System.EventArgs e)
		{
			Color_Click();
		}

		private void dimShowMenuItem_Click(object sender, System.EventArgs e)
		{
			ChangeDimensionsVisiblity();
		}

		private void textsShowMenuItem_Click(object sender, System.EventArgs e)
		{
			ChangeTextsVisiblity();
		}

		private void layersShowMenuItem_Click(object sender, System.EventArgs e)
		{
			SetLayList();
			lForm.ShowDialog();
		}

		private void orbitMenuItem_Click(object sender, System.EventArgs e)
		{
			Go3dOrbit(! this.Orb3D.Visible);
		}

		private void splitter1_SplitterMoved(object sender, System.Windows.Forms.SplitterEventArgs e)
		{
			this.DoResize();
			this.cadPictBox.Invalidate();
		}	
	
		#region Help
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Visible">CADLayer.Visible</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>
		#endregion
		public void SetVisibleLayer(bool visible, int index)
		{
			cadImage.SetLayerVisible(index, visible, lForm.detInvalidate);			
			InitialNewPrintPages();			
			cadPictBox.Invalidate();			
		}
		#region Help
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Frozen">CADLayer.Frozen</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers">layers</see> collection</param>
		#endregion
		public void SetFreezeLayer(bool visible, int index)
		{
			cadImage.SetLayerFreeze(index, visible, lForm.detInvalidate);			
			InitialNewPrintPages();
			cadPictBox.Invalidate();			
		}
		#region Help
		/// <summary>
		/// Sets layer color by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="color">Setting <see cref="System.Drawing.Color">color</see></param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>
		#endregion
		public void SetColorLayer(Color color, int index)
		{
			cadImage.SetLayerColor(index, color, lForm.detInvalidate);				
			InitialNewPrintPages();
			cadPictBox.Invalidate();
		}

		private void InitialNewPrintPages()
		{
			this.prtForm.Image = this.cadImage;
			this.prtForm.CreateNewPages();	
		}

		private void ListSelect(ArrayList item, ListEventSelectArgs e)
		{
			ListItem itm1 =((ListItem)lForm.LayerList.Items[e.ItemIndex]);
			int index = e.ItemIndex / lForm.LayerList.Columns.Count;
			CADImportFace.SelectLayer(index, this.cadImage);
			switch(itm1.ImageIndex)
			{
				case 7:
					itm1.ImageIndex = 8;	
					SetVisibleLayer(false, index);				
					break;
				case 8:
					itm1.ImageIndex = 7;										
					SetVisibleLayer(true, index);
					break;
				case 4:
					itm1.ImageIndex = 3;
					SetFreezeLayer(true, index);
					break;
				case 3:
					itm1.ImageIndex = 4;
					SetFreezeLayer(false, index);
					break;
			}
			if(itm1.DrawColorRect)
			{
				this.dlgSetColor.Color = itm1.BackColor;
				if(this.dlgSetColor.ShowDialog() == DialogResult.OK)
				{
					SetColorLayer(this.dlgSetColor.Color, index);	
					itm1.BackColor = this.dlgSetColor.Color;
					this.lForm.LayerList.InvalidateCell(e.ItemIndex);			
				}
			}
			lForm.LayerList.InvalidateCell(e.ItemIndex);
		}		

		private void printprevItem_Click(object sender, System.EventArgs e)
		{
			dlgPrintPrev.Document = cadImage.Print(false, false);
			dlgPrintPrev.ShowDialog();
			dlgPrintPrev.Document.Dispose();
		}

		private void btnPrintPrevCust_Click(object sender, System.EventArgs e)
		{
			prtForm.ShowDialog();
		}

		private void mnSaveAsDXF_Click(object sender, System.EventArgs e)
		{
			if(this.cadImage == null) return;
			if(this.dlgSaveDXF.ShowDialog() != DialogResult.OK) return;
			SaveAsDXF(this.dlgSaveDXF.FileName);
		}

		private void SaveAsDXF(string fName)
		{
			#region Export
#if Export
			if(cadImage == null) return;
			if(cadImage is CADRasterImage) 
			{
				return;
			}
			CADImport.Export.DirectCADtoDXF.CADtoDXF vExp = new CADImport.Export.DirectCADtoDXF.CADtoDXF(cadImage);
			vExp.SaveToFile(fName);
#else
				MessageBox.Show("Not supported in current version!", "CADImport .Net");
#endif
			#endregion Export

			#region ExportOverMetafile 
			/*#region Export
#if Export
				if(! (this.FCADImage is RasterImage.CADRasterImage)) 
					this.FCADImage.SaveAsDXF(fName, this.cadPictBox.Handle, actForm);
				else
					MessageBox.Show("This operation is not supported for current file format", "CADImport .Net");
#else
					MessageBox.Show("Not supported in current version!", "CADImport .Net");
#endif
			#endregion Export*/
			#endregion
		}

		private void miFloatLic_Click(object sender, System.EventArgs e)
		{
			#region protect
#if floatprotect
				floatLicFrm.ShowDialog();
#endif
			#endregion
		}

		private void miMerge_Click(object sender, System.EventArgs e)
		{
			if(this.cadImage == null)
				this.LoadEmptyImage();
			this.mergeForm.ShowDialog();							
		}
	}

	internal abstract class ApplicationConstants
	{
		public static readonly string defaultstr;
		public static readonly string languagepath;
		public static readonly string loadfilestr;
		public static readonly string msgBoxDebugCaption;
		public static readonly string sepstr;
		public static readonly string notsupportedstr;
		public static readonly string notsupportedextstr;
		public static readonly string appnamestr;
		public static readonly string sepstr2;
		public static readonly string jpgextstr;
		public static readonly string bmpextstr;
		public static readonly string tiffextstr;
		public static readonly string gifextstr;
		public static readonly string emfextstr;
		public static readonly string wmfextstr;
		public static readonly string pngextstr;			
		public static readonly string dxfextstr;		
		public static readonly string languagestr;
		public static readonly string languageIDstr;
		public static readonly string backcolorstr;
		public static readonly string blackstr;
		public static readonly string showentitystr;
		public static readonly string drawmodestr;
		public static readonly string truestr;
		public static readonly string shxpathcnstr;
		public static readonly string installstr;
		public static readonly string sepstr3;
		public static readonly string typelcstr;
		public static readonly string floatingstr;
		public static readonly string hoststr;
		public static readonly string portstr;
		public static readonly string appconst;
		public static readonly string blackstr2;
		public static readonly string whitestr;
		public static readonly string registerstr;
		public static readonly string headstr1;
		public static readonly string headstr2;
		public static readonly string headstr3;
		public static readonly string headstr4;
		public static readonly string namestr;
		public static readonly string visiblestr;
		public static readonly string freezestr;
		public static readonly string colorstr;		
		public static readonly string zerostr;
		public static readonly string hundredstr;

		static ApplicationConstants()
		{
			defaultstr = "Default";
			languagepath = "LanguagePath";
			loadfilestr = "Loading file...";
			msgBoxDebugCaption = "Debug application message";
			sepstr = " - ";
			notsupportedstr = "not supported";
			notsupportedextstr = "Not supported in current version!";
			appnamestr = "CADImportNet Demo";
			sepstr2 = " : ";			
			jpgextstr = ".JPG";
			bmpextstr = ".BMP";
			tiffextstr = ".TIFF";
			gifextstr = ".GIF";
			emfextstr = ".EMF";
			wmfextstr = ".WMF";
			pngextstr = ".PNG";			
			dxfextstr = ".DXF";			
			languagestr = "Language";
			languageIDstr = "LanguageID";
			backcolorstr = "BackgroundColor";
			blackstr = "BLACK";
			showentitystr = "ShowEntity";
			drawmodestr = "drawMode";
			truestr = "TRUE";
			shxpathcnstr = "SHXPathCount";
			installstr = "Install";
			sepstr3 = ";";
			typelcstr = "Type_license";
			floatingstr = "floating";
			hoststr = "Host";
			portstr = "Port";
			appconst = "Application";
			blackstr2 = "Black";
			whitestr = "White";
			registerstr = "register";
			headstr1 = "Head1";
			headstr2 = "Head2";
			headstr3 = "Head3";
			headstr4 = "Head4";
			namestr = "Name";
			visiblestr = "Visible";
			freezestr = "Freeze";
			colorstr = "Color";		
			zerostr = "0";
			hundredstr = "100";
		}
	}
}
